/***************************************************************************
                          commandentry.cpp  -  description
                             -------------------
    begin                : Thu Jul 20 2000
    copyright            : (C) 2000-2001 by Eggert Ehmke
    email                : eggert.ehmke@berlin.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include <kapplication.h>
#include <kaction.h>
#include <kdebug.h>
#include <qmessagebox.h>

#include "commandentry.h"
#include "editdialog.h"

CommandEntry& CommandEntry::operator = (const CommandEntry& entry)
{
	m_id = entry.m_id;
	m_name = entry.m_name;
	m_path = entry.m_path;
  m_waitForCommand = entry.m_waitForCommand;
  return *this;
}

bool CommandEntry::exec (ConfigElem* pelem, const char* header, const char* body)
{
	kdDebug () << "CommandEntry::exec ();" << endl;
	return exec (m_path, pelem, header, body);
}

bool CommandEntry::exec (const QString& path, ConfigElem* pelem, const char* header, const char* body)
{
	kdDebug () << "CommandEntry::exec (" << path << ");" << endl;
	QString cmd = path;
	int pos;
	if (pelem)
	{
		if ((pos = cmd.find ("<user>")) >= 0)
			cmd.replace (pos, strlen ("<user>"), pelem->m_url.user ());
		if ((pos = cmd.find ("<server>")) >= 0)
			cmd.replace (pos, strlen ("<server>"), pelem->m_url.host());
		if ((pos = cmd.find ("<passwd>")) >= 0)
			cmd.replace (pos, strlen ("<passwd>"), pelem->m_url.pass());
	}
  else
  {
		if ((pos = cmd.find ("<user>")) >= 0)
			cmd.remove (pos, strlen ("<user>"));
		if ((pos = cmd.find ("<server>")) >= 0)
			cmd.remove (pos, strlen ("<server>"));
		if ((pos = cmd.find ("<passwd>")) >= 0)
			cmd.remove (pos, strlen ("<passwd>"));
  }
	if (header && (pos = cmd.find ("<header>")) >= 0)
	{
		FILE* tmp = fopen ("/tmp/spamheader", "w");
		fputs (header, tmp);
		fclose (tmp);
		cmd.replace (pos, strlen ("<header>"), "/tmp/spamheader");
	}
	if (body && (pos = cmd.find ("<body>")) >= 0)
	{
		FILE* tmp = fopen ("/tmp/spambody", "w");
		fputs (body, tmp);
		fclose (tmp);
		cmd.replace (pos, strlen ("<body>"), "/tmp/spambody");
	}
  int result = 0;
  if (m_waitForCommand) {
    //execute the command and wait for it to return
    FILE* output = popen (cmd, "r");
  	char buffer [81];
  	QString str;
  	while (fgets (buffer, 80, output))
  	{
  		str += buffer;
  	}
 		KApplication::setOverrideCursor (Qt::arrowCursor);
  	result = pclose (output);
  	if (!str.isEmpty ())
  	{
  		EditDialog dlg (NULL, cmd, str, false);
  		dlg.exec ();
  	}
  	kdDebug () << cmd << " returned " << result << endl;
  }
  else
  {
    //execute the command and don't wait for it to return
    int pid;
    if ((pid = fork()) == 0)
    {
      // child processing; don't do any gui stuff here !
      system (cmd);
      // we must terminate the child process
      exit (0);
      // the bad thing is, we never know if the command failed. We would need some pipe.
    }
    else if (pid == -1)
    {
      EditDialog dlg (NULL, cmd, "The command failed to execute", false);
      dlg.exec ();
    }
  }
  KApplication::restoreOverrideCursor ();
  return (result == 0);
}

