/***************************************************************************
                          commandlist.cpp  -  description
                             -------------------
    begin                : Wed Oct 31 2001
    copyright            : (C) 2001 by Eggert Ehmke
    email                : eggert.ehmke@berlin.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include "commandentry.h"
#include "commandlist.h"

CommandList::CommandList()
{
	setAutoDelete (true);
}

CommandList::~CommandList()
{
}

void CommandList::setCombo (QComboBox* combo, int nIndex)
{
	combo->clear ();
	for (CommandEntry* pElem = first(); pElem; pElem = next())
	{
		combo->insertItem (pElem->m_name);
	}
	if (nIndex >= 0)
	{
		at (nIndex);
		combo->setCurrentItem (nIndex);
	}
}

CommandList& CommandList::operator = (CommandList& list)
{
	clear ();
	int nIndex = list.at ();
	for (CommandEntry* item = list.first (); item; item = list.next ())
	{
		append (new CommandEntry (item));		
	}
	at (nIndex);
	return *this;
}

bool CommandList::exec (unsigned int id, ConfigElem* pelem, const char* header, const char* body)
{
	bool result = false;
	CommandEntry* test = new CommandEntry (NULL, NULL, id, true);
	int pos = find (test);
	if (pos >= 0)
	{
		result = at (pos)->exec (pelem, header, body);
	}
	else
		kdError () << "No ID " << id << " found in commandlist." << endl;
	delete test;
	return result;
}

bool CommandList::exec (unsigned int pos, ConfigElem* pelem)
{
	// command may have bee deleted
	if (pos < count ())
		return at (pos)->exec (pelem);
	else
		return false;
}

int CommandList::compareItems (QCollection::Item item1, QCollection::Item item2 )
{
	CommandEntry* entry1 = (CommandEntry*) item1;
	CommandEntry* entry2 = (CommandEntry*) item2;
	if (entry1->m_id > entry2->m_id)
		return 1;
	else if (entry1->m_id == entry2->m_id)
		return 0;
	else return -1;
}

QCollection::Item CommandList::newItem (QCollection::Item d)
{
	return new CommandEntry ((CommandEntry*)d);
}

void CommandList::readOptions (KConfig* config)
{
	config->setGroup ("usercommands");
	bool goon = true;
	int i = 0;
	while (goon)
	{
		QString cmd (config->readEntry (QString ("cmd%1").arg(i)));
		if (!cmd.isEmpty ())
		{
			QString path = config->readEntry (QString("path%1").arg(i));
			int id = config->readNumEntry (QString("id%1").arg(i));
      bool modal = config->readBoolEntry (QString("modal%1").arg(i), true);
      append (new CommandEntry (cmd, path, id, modal));
			i++;
		}
		else
			goon = false;
	}
	if (i > 0)
		at (0);
}

void CommandList::saveOptions (KConfig* config)
{
	kdDebug () << "CommandList::saveOptions" << endl;
	config->setGroup ("usercommands");
	for (unsigned int i = 0; i < count (); i++)
	{
		config->writeEntry (QString("cmd%1").arg(i), at(i)->m_name);
		config->writeEntry (QString("path%1").arg(i), at(i)->m_path);
		config->writeEntry (QString("id%1").arg(i), at(i)->m_id);
		config->writeEntry (QString("modal%1").arg(i), at(i)->m_waitForCommand);
	}
}
