/***************************************************************************
                          ConfigElem.cpp  -  description
                             -------------------
    begin                : Tue May 9 2000
    copyright            : (C) 2000-2001 by Eggert Ehmke
    email                : eggert.ehmke@berlin.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include "configelem.h"

static const char scramble1 [50] = "C6FDC7A1EDFBB6FEE3DBF5BEBAEFDDF7ABC6FDC7A1EDFBB6";
static const char hexstr [17] = "0123456789ABCDEF";

int hexbyt (const char c)
{
	if (c >= '0' && c <= '9')
		return c - '0';
	else
		return c - 'A' + 10;	
}

const QString crypt (const KURL& url)
{
	char result [50];
	char scramble2 [50];
	QString hexresult;

	memset (result, 0, 50);
	memset (scramble2, 0, 50);
  int pos = url.pass().length () + 1;
  unsigned int free = 50 - pos;
  if (url.user().length() <= free)
  {
	  strcpy (&scramble2[pos], url.user ());
    pos += url.user().length();
    free -= url.user().length();
  }
  else
  {
	  memcpy (&scramble2[pos], url.user ().latin1(), free);
    free = 0;
  }
  if (url.host ().length() <= free)
  {
	  strcpy (&scramble2[pos], url.host ());
    pos += url.host().length();
    free -= url.host().length();
  }
  else
  {
	  memcpy (&scramble2[pos], url.host().latin1(), free);
    free = 0;
  }

  memcpy (result, url.pass().latin1(), url.pass().length());
	for (int i = 0; i <= 31; i++)
	{
  	result [i] = (char)(result[i]  ^ (scramble1 [i] ^ scramble2 [i]));
  	hexresult += hexstr [result [i] / 16];
  	hexresult += hexstr [result [i] % 16];
	}
	return hexresult;
}

const QString decrypt (const QString& pass)
{
  char result [50];

	memset (result, 0, 50);
	int i;
	for (i = 0; i <= 31; i++)
	{
		result [i] = (char)hexbyt (pass[i*2]) * 16 + hexbyt (pass[i*2+1]);
  	result [i] = (char)(result[i] ^ scramble1 [i]);
	}
	return result;
}

ConfigElem::ConfigElem():
	m_pViewItem (NULL),
	m_pshowrecord (NULL)
{
	m_url.setProtocol ("pop3");
	m_url.setPort (110);
	m_pshowrecord = new ShowRecord ();
}

ConfigElem::ConfigElem (const ConfigElem* pElem):
	m_bActive (pElem->m_bActive),
	m_pViewItem (pElem->m_pViewItem),
	m_strAccount (pElem->m_strAccount),
	m_url (pElem->m_url),
	m_pshowrecord (pElem->m_pshowrecord)
{	
//	kdDebug () << "ConfigElem::ConfigElem (const ConfigElem* pElem)" << endl;
}

ConfigElem::ConfigElem (const QString& account, const KURL& url, bool active):
	m_bActive (active),
	m_pViewItem (NULL),
	m_strAccount (account),
	m_url (url),
	m_pshowrecord (NULL)
{
//	kdDebug () << "ConfigElem::ConfigElem (const QString& account, const KURL& url, bool active)" << endl;
	m_pshowrecord = new ShowRecord ();
}

ConfigElem::ConfigElem (const QString& account):
	m_bActive (false),
	m_pViewItem (NULL),
	m_strAccount (account)
{
//	kdDebug () << "ConfigElem::ConfigElem (const QString& account)" << endl;
	m_pshowrecord = new ShowRecord ();
}

ConfigElem::~ConfigElem()
{
	// do not delete m_pshowrecord here
}

ConfigElem& ConfigElem::operator = (const ConfigElem& right)
{
//	kdDebug () << "ConfigElem::operator = (const ConfigElem& right)" << endl;
	m_bActive = right.m_bActive;
	m_pViewItem = right.m_pViewItem;
	m_strAccount = right.m_strAccount;
	m_url = right.m_url;
	m_pshowrecord = right.m_pshowrecord;
	return *this;
}

void ConfigElem::saveOptions (QDomDocument& doc, QDomElement& parent, bool bSavePassword)
{
//  kdDebug () << "ConfigElem::saveOptions: " << m_strAccount << endl;
  parent.setAttribute ("account", m_strAccount);
	parent.setAttribute ("server", m_url.host ());
	parent.setAttribute ("protocol", m_url.protocol ());
	parent.setAttribute ("port", m_url.port ());
	parent.setAttribute ("user", m_url.user ());
	if (bSavePassword)
		parent.setAttribute ("password", crypt (m_url));
	else
		parent.setAttribute ("password", "");
	parent.setAttribute ("active", m_bActive);

	m_pshowrecord->saveOptions (doc, parent);
}

void ConfigElem::readOptions (QDomElement& parent, bool bSavePassword)
{
	m_strAccount = parent.attribute ("account");
	m_url.setHost (parent.attribute ("server"));
	m_url.setProtocol (parent.attribute ("protocol", "pop3"));
	m_url.setPort (parent.attribute ("port", "110").toInt());
	m_url.setUser (parent.attribute ("user"));
	if (bSavePassword)
		m_url.setPass (decrypt (parent.attribute ("password", "")));
	m_bActive = parent.attribute ("active", "1").toInt();
	
	m_pshowrecord->readOptions (parent);
//  kdDebug () << "ConfigElem count = " << count () << endl;
}

ShowRecordElem* ConfigElem::current ()
{
	return m_pshowrecord->current();
}

ShowRecordElem* ConfigElem::first ()
{
	return m_pshowrecord->first();
}

ShowRecordElem* ConfigElem::next ()
{
	return m_pshowrecord->next();
}

ShowRecordElem* ConfigElem::prev ()
{
	return m_pshowrecord->prev();
}

ShowRecordElem* ConfigElem::at (int i)
{
	return m_pshowrecord->at(i);
}

void ConfigElem::append (ShowRecordElem* elem)
{
	m_pshowrecord->append(elem);
}

void ConfigElem::removeRef (ShowRecordElem* elem)
{
	m_pshowrecord->removeRef(elem);
}

int ConfigElem::count ()
{
	return m_pshowrecord->count();
}

void ConfigElem::applyFilters ()
{
  m_pshowrecord->applyFilters();
}

