/***************************************************************************
                          configlist.cpp  -  description
                             -------------------
    begin                : Tue May 9 2000
    copyright            : (C) 2000-2001 by Eggert Ehmke
    email                : eggert.ehmke@berlin.de

    26 Sep 2002 - Allow for columns to be hidden. Allistar Melville
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include <qfile.h>

#include <kconfig.h>
#include <ksavefile.h>
#include <kapplication.h>
#include <kstandarddirs.h>
#include <kaudioplayer.h>
#include <kdebug.h>

#include "configlist.h"
#include "commandlist.h"
#include "filter.h"

ConfigList::ConfigList()
{
	setAutoDelete (true);
	m_ConfigFileName = locateLocal ("config", "kshowmail.xml");
}

int ConfigList::compareItems (QCollection::Item item1, QCollection::Item item2)
{
	ConfigElem* p1 = (ConfigElem*)item1;
	ConfigElem* p2 = (ConfigElem*)item2;
	return strcmp (p1->m_strAccount, p2->m_strAccount);
}

QCollection::Item ConfigList::newItem (QCollection::Item item)
{
	return new ConfigElem ((ConfigElem*)item);
}

void ConfigList::saveOptions ()
{
	kdDebug () << "ConfigList::saveOptions" << endl;
	QDomDocument doc ("KShowmail");
	QDomElement main = doc.createElement ("config");
	doc.appendChild (main);
	
	QDomElement gen = doc.createElement ("General_Options");
	gen.setAttribute ("savepassword", m_bSavePassword);
	gen.setAttribute ("confirmdelete", m_bConfirmDelete);
	gen.setAttribute ("inittimer", m_nInitTimer);
	gen.setAttribute ("intervaltimer", m_nIntervalTimer);
	gen.setAttribute ("pop3timer", m_nPop3Timer);
	gen.setAttribute ("maxmailsize", m_nMaxMailSize);
  gen.setAttribute ("beep", m_bBeep);
  gen.setAttribute ("sound", m_bSound);
  gen.setAttribute ("external", m_bExternalProg);
  gen.setAttribute ("soundfile", m_strSoundFile);
  gen.setAttribute ("externalprog", m_nExternalProg);
  gen.setAttribute ("startminimized", m_bStartMinimized);
  gen.setAttribute ("terminate", m_bTerminate);
  gen.setAttribute ("minimize", m_bMinimize);
  gen.setAttribute ("keepnew", m_bKeepNew);
  gen.setAttribute ("showmessage", m_bShowMessage);
  gen.setAttribute ("showmainwindow", m_bShowMainWindow);
 	//changes by AM to remember the columns that are turned on
 	int i;
 	for(i=0;i<6;i++)
 	 	gen.setAttribute(QString("accountColumn%1").arg(i), m_shownAccountColumns[i]);
 	for(i=0;i<8;i++)
 	 	gen.setAttribute(QString("mailColumn%1").arg(i), m_shownMailColumns[i]);
  gen.setAttribute ("allowhtml", m_bAllowHTML);
  gen.setAttribute ("confirmclose", m_bConfirmClose);
  main.appendChild (gen);
	
	QDomElement accounts = doc.createElement ("accounts");
	i = 0;
  for (ConfigElem* pElem = first (); pElem; pElem = next())
	{
		QDomElement account = doc.createElement (QString ("account%1").arg(i++));
		pElem->saveOptions (doc, account, m_bSavePassword);
		accounts.appendChild (account);
	}
	main.appendChild (accounts);

  QCString str = doc.toCString ();
	QString cachefilename = locateLocal ("config", "kshowmail.xml");
  KSaveFile file (cachefilename, 0600);
  if (file.status() != 0)
  {
    kdError () << "Couldn't save cache. " << strerror(file.status());
    return;
  }

  file.file()->writeBlock (str.data(), str.length());
  if (!file.close())
  {
    kdError () << "Couldn't save cache. " << strerror(file.status());
    return;
  }
}

void ConfigList::readOptions (QListView* list)
{
 	int i;
  QFile file (m_ConfigFileName);
  if (!file.open (IO_ReadOnly))
  {
  	kdError () << "Can't open " << m_ConfigFileName << endl;
    kdError () << "setting defaults" << endl;
	  m_bSavePassword = false;
	  m_bConfirmDelete = true;
  	m_nInitTimer = 0;
  	m_nIntervalTimer = 0;
    m_nPop3Timer = 30;
  	m_nMaxMailSize = 10;
  	m_bBeep = true;
  	m_bSound = false;
  	m_bExternalProg = false;
  	m_strSoundFile = "kshowmail/sounds/newmail.wav";
  	m_nExternalProg = -1;
  	m_bStartMinimized = false;
  	m_bTerminate = false;
  	m_bMinimize = false;
  	m_bKeepNew = false;
  	m_bShowMessage = true;
  	m_bShowMainWindow = false;
    for(i=0;i<6;i++)
      m_shownAccountColumns[i] = true;
    for(i=0;i<8;i++)
      m_shownMailColumns[i] = true;
    m_bAllowHTML = false;
    m_bConfirmClose = false;
    return;
  }

  QDomDocument doc ("KShowmail");
  doc.setContent (&file);

  QDomElement docElem = doc.documentElement();
  if (docElem.isNull())
  	kdError () << "Showrecord::parse : can't parse " << m_ConfigFileName << endl;
	
	QDomElement gen = docElem.namedItem ("General_Options").toElement();
  kdDebug () << "setting general options" << endl;
  m_bSavePassword = (bool)gen.attribute ("savepassword", "0").toInt();
  m_bConfirmDelete = (bool)gen.attribute ("confirmdelete", "1").toInt();
 	m_nInitTimer = gen.attribute ("inittimer", "0").toInt();
 	m_nIntervalTimer = gen.attribute ("intervaltimer", 0).toInt();
 	m_nPop3Timer = gen.attribute ("pop3timer", "30").toInt();
 	m_nMaxMailSize = gen.attribute ("maxmailsize", "10").toInt();
 	m_bBeep = (bool)gen.attribute ("beep", "1").toInt();
 	m_bSound = (bool)gen.attribute ("sound", "0").toInt();
 	m_bExternalProg = (bool)gen.attribute ("external", "0").toInt();
 	m_strSoundFile = gen.attribute ("soundfile", ::locate ("data", "kshowmail/sounds/newmail.wav"));
 	m_nExternalProg = gen.attribute ("externalprog", "-1").toInt();
 	m_bStartMinimized = (bool)gen.attribute ("startminimized", "0").toInt();
 	m_bTerminate = (bool)gen.attribute ("terminate", "0").toInt();
 	m_bMinimize = (bool)gen.attribute ("minimize", "0").toInt();
 	m_bKeepNew = (bool)gen.attribute ("keepnew", "0").toInt();
 	m_bShowMessage = (bool)gen.attribute ("showmessage", "1").toInt();
 	m_bShowMainWindow = (bool)gen.attribute ("showmainwindow", "0").toInt();
 	//changes by AM to retrieve which columns are hidden or shown
 	for(i=0;i<6;i++)
 	 	m_shownAccountColumns[i] = (bool)gen.attribute(QString("accountColumn%1").arg(i), "1").toInt();
 	for(i=0;i<8;i++)
 	m_shownMailColumns[i] = (bool)gen.attribute(QString("mailColumn%1").arg(i), "1").toInt();
  m_bAllowHTML = (bool)gen.attribute ("allowhtml", "0").toInt();
  m_bConfirmClose = (bool)gen.attribute ("confirmclose", "0").toInt();

	QDomElement accounts = docElem.namedItem ("accounts").toElement();
  QDomNode n = accounts.firstChild();
  while (!n.isNull())
  {
    QDomElement e = n.toElement();
		ConfigElem* elem = new ConfigElem();
		elem->readOptions (e, m_bSavePassword);
		inSort (elem);
    n = n.nextSibling();
	}
	
	setList (list);

	file.close ();
}

void ConfigList::setCombo (QComboBox* combo)
{
	combo->clear ();
	int nIndex = at ();
	for (ConfigElem* pElem = first(); pElem; pElem = next())
		combo->insertItem (pElem->m_strAccount);
	if (nIndex >= 0)
	{
		at (nIndex);
		combo->setCurrentItem (nIndex);
	}
}

void ConfigList::setList (QListView* list)
{
	QPixmap pix (::locate ("data", "kshowmail/pics/ok.png"));
	list->clear ();
	int nIndex = at ();
	QListViewItem* last = NULL;
	for (ConfigElem* pElem = first(); pElem; pElem = next())
	{
		last = pElem->m_pViewItem = new QListViewItem (list, last, "", pElem->m_strAccount, pElem->m_url.host(), pElem->m_url.user(), "?");
		if (pElem->m_bActive)
			pElem->m_pViewItem->setPixmap (0, pix);
	}

	if (nIndex >= 0)
	{
		at (nIndex);
//		list->setCurrentItem (nIndex);
	}
}


ConfigList& ConfigList::operator = (ConfigList& list)
{
	clear ();
	int nIndex = list.at ();
	for (ConfigElem* item = list.first(); item; item=list.next())
	{
		append (new ConfigElem (item));
	}
	at (nIndex);

	m_bSavePassword = list.m_bSavePassword;
	m_bConfirmDelete = list.m_bConfirmDelete;
	m_nInitTimer = list.m_nInitTimer;
	m_nIntervalTimer = list.m_nIntervalTimer;
  m_nPop3Timer = list.m_nPop3Timer;
	m_nMaxMailSize = list.m_nMaxMailSize;
	m_bBeep = list.m_bBeep;
	m_bSound = list.m_bSound;
	m_bExternalProg = list.m_bExternalProg;
	m_strSoundFile = list.m_strSoundFile;
	m_nExternalProg = list.m_nExternalProg;
	m_bStartMinimized = list.m_bStartMinimized;
	m_bTerminate = list.m_bTerminate;
	m_bMinimize = list.m_bMinimize;
	m_bKeepNew = list.m_bKeepNew;
	m_bShowMessage = list.m_bShowMessage;
	m_bShowMainWindow = list.m_bShowMainWindow;
	m_ConfigFileName = list.m_ConfigFileName;
  int i;
  for(i=0;i<6;i++)
    m_shownAccountColumns[i] = list.m_shownAccountColumns[i];
  for(i=0;i<8;i++)
    m_shownMailColumns[i] = list.m_shownMailColumns[i];
  m_bAllowHTML = list.m_bAllowHTML;
  m_bConfirmClose = list.m_bConfirmClose;
  return *this;
}

ConfigList::ConfigList (ConfigList& list):
  QPtrList<ConfigElem>()
{
	int nIndex = list.at ();

	setAutoDelete (true);
	for (ConfigElem* item = list.first(); item != NULL; item=list.next())
	{
		append (new ConfigElem (item));
	}

	at (nIndex);

	m_bSavePassword = list.m_bSavePassword;
	m_bConfirmDelete = list.m_bConfirmDelete;
	m_nInitTimer = list.m_nInitTimer;
	m_nIntervalTimer = list.m_nIntervalTimer;
  m_nPop3Timer = list.m_nPop3Timer;
	m_nMaxMailSize = list.m_nMaxMailSize;
	m_bBeep = list.m_bBeep;
	m_bSound = list.m_bSound;
	m_bExternalProg = list.m_bExternalProg;
	m_strSoundFile = list.m_strSoundFile;
	m_nExternalProg = list.m_nExternalProg;
	m_bStartMinimized = list.m_bStartMinimized;
	m_bTerminate = list.m_bTerminate;
	m_bMinimize = list.m_bMinimize;
	m_bKeepNew = list.m_bKeepNew;
	m_bShowMessage = list.m_bShowMessage;
	m_bShowMainWindow = list.m_bShowMainWindow;
	m_ConfigFileName = list.m_ConfigFileName;
  int i;
  for(i=0;i<6;i++)
    m_shownAccountColumns[i] = list.m_shownAccountColumns[i];
  for(i=0;i<8;i++)
    m_shownMailColumns[i] = list.m_shownMailColumns[i];
  m_bAllowHTML = list.m_bAllowHTML;
  m_bConfirmClose = list.m_bConfirmClose;
	setAutoDelete (true);
}

bool ConfigList::setItem (const char* item)
{
	int nPos = at ();
	ConfigElem* pActive = new ConfigElem (item);
	bool result = (find (pActive) >= 0);
	delete pActive;
	if (result)
		return true;
	else
	{
		at (nPos);
		return false;
	}
}

void ConfigList::beep ()
{
	if (m_bBeep)
		kapp->beep ();
}

void ConfigList::playSound ()
{
	if (m_bSound)
  	playSound (m_strSoundFile);
}

void ConfigList::playSound (const char* file)			
{
	KAudioPlayer::play(file);
}

void ConfigList::startProgram (CommandList* pCmdList, ConfigElem* pElem)
{
	if (m_bExternalProg)
	{
		kdDebug () << "ConfigList::startProgram; n = " << m_nExternalProg << endl;
		pCmdList->exec (m_nExternalProg, pElem);
	}
}

void ConfigList::nextHighLight ()
{
//	kdDebug () << "ConfigList::nextHighLight" << endl;
	if (current ())
	{
		if (current()->current())
			current()->next();
		if (current()->current() == NULL)
		{
			next();
			if (current())
				current()->first();
		}
	}
	skipHighLight ();
}

void ConfigList::skipHighLight ()
{
//	kdDebug () << "ConfigList::skipHighLight" << endl;
  while (current())
  {
  	if (current() && !current()->m_bActive)
    {
      next();
			if (current())
				current()->first();
      continue;
    }
		if (current()->current())
		{
			if (current()->current()->viewItem() &&
					current()->current()->viewItem()->isSelected())
			{
				return;
			}
			current()->next();
		}
		if (current()->current() == NULL)
		{
			next();
			if (current())
				current()->first();
		}
  }
}

void ConfigList::applyFilters ()
{
  if (Filter::_status != Filter::off)
  {
    for (ConfigElem* pElem = first(); pElem; pElem = next())
    {
      if (pElem->m_bActive)
      {
        pElem->applyFilters (); 
      }
    }
  }
}
