/***************************************************************************
                          kshowmail.cpp  -  description
                             -------------------
    begin                : Sat May  6 12:13:57 MEST 2000
    copyright            : (C) 2000-2002 by Eggert Ehmke
    email                : eggert.ehmke@berlin.de

    26 Sep 2002 - Allow for columns to be hidden. Allistar Melville
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// includes for system functions
#include <stdlib.h>

// include files for QT
#include <qdir.h>
#include <qstrlist.h>
#include <qpainter.h>
#include <qdatetime.h>

// include files for KDE
#include <klocale.h>
#include <kiconloader.h>
#include <kmenubar.h>
#include <kpopupmenu.h>
#include <kstatusbar.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kcombobox.h>
#include <kmessagebox.h>
#include <kuniqueapplication.h>
#include <kstandarddirs.h>
#include <kwin.h>
#include <krun.h>
#include <kpassdlg.h>
#include <kdebug.h>
#include <kio/global.h>

// application specific includes
#include "kshowmail.h"
#include "kshowmailview.h"
#include "kshowmaildoc.h"
#include "optiondialog.h"
#include "serverdialog.h"
#include "editdialog.h"
#include "commanddialog.h"
#include "filterdialog.h"
#include "alertdialog.h"
#include "showlistviewitem.h"

using namespace KIO;

KAboutData* KShowMailApp::m_pAbout;

KShowMailApp::KShowMailApp()
{
  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
	initActions();
  initStatusBar();
	initMenuBar();
  initToolBar();
  initDocument();
  initView();
	
  m_state = idle;
  m_nRetry = 0;
	m_pTimer = new QTimer (this);
	m_pPop3Timer = new QTimer (this);
	m_pAlertDialog = NULL;
  m_pMessagebox = NULL;
	m_pDockWindow = new KShowMailDock (this);
	m_pPop3 = NULL;
	
  readOptions();

  connect (m_pTimer, SIGNAL (timeout ()), SLOT (slotRefresh ()));
	connect (m_pPop3Timer, SIGNAL (timeout ()), SLOT (slotPop3Timeout ()));
	connect (m_pView->m_pListAccounts, SIGNAL (currentChanged(QListViewItem*)), SLOT(slotAccountActivated(QListViewItem*)));
  connect (m_pView, SIGNAL (signalActiveChanged()), SLOT (slotRefreshView()));

	// this starts the refresh timer
	restartTimer ();
	// this starts the one-second-interval timer
	startTimer (1000);
	m_ConfigList.setAutoDelete (true);
  ///////////////////////////////////////////////////////////////////
  // disable menu and toolbar items at startup
}

KShowMailApp::~KShowMailApp()
{
	delete m_pTimer;
	delete m_pPop3Timer;
	delete m_pDockWindow;
//  delete m_pAlertDialog;
//  delete m_pMessagebox;
//	if (m_pPop3)
//		m_pPop3->kill();
}

void KShowMailApp::timerEvent (QTimerEvent *)
{
	if (m_nSecondsToGo)
	{
		QTime time;
		time = time.addSecs (m_nSecondsToGo--);
		QCString msg (i18n("Autorefresh: %1").arg (time.toString()));
  	statusBar()->changeItem(msg, 3);
	}
	else
  	statusBar()->changeItem("", 3);

  // if busy, animate the traybar
  if ((m_state >= waitIndexResult) && (m_state <= waitDeleteCont))
  {
    static int color = 0;
    color = (color+1)%2;
    switch (color)
    {
      case 0:
        m_pDockWindow->clear ();
        break;
      default:
        m_pDockWindow->drawText ("?", Qt::red);
    }
  }
}

KshowmailDoc *KShowMailApp::getDocument() const
{
  return m_pDoc;
}

void KShowMailApp::initActions ()
{
	// file menu
	m_actionSave = new KAction(i18n("&Save"), "filesave", 0, this, SLOT(slotSaveOptions()), actionCollection(), "save");
  m_actionQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());

	// action menu
	m_actionRefresh = new KAction(i18n("&Refresh messages"), "reload", Key_F5, this, SLOT(slotRefresh()), actionCollection(), "refresh");
	m_actionForcedRefresh = new KAction(i18n("Refresh &all messages"), "reload", SHIFT+Key_F5, this, SLOT(slotForcedRefresh()), actionCollection(), "refresh_all");
	m_actionShowHeader = new KAction(i18n("Show &header of highlighted messages"), "letter-closed", Key_F2, this, SLOT(slotShowHeader()), actionCollection(), "show");
	m_actionShowMessage = new KAction(i18n("Show &complete highlighted messages"), "letter-open", Key_F3, this, SLOT(slotShowMessage()), actionCollection(), "show_complete");
	m_actionDelete = new KAction(i18n("&Delete highlighted messages"), "eraser", Key_Delete, this, SLOT(slotDelete()), actionCollection(), "delete");
	m_actionComplain = new KAction(i18n("&Send complain mail"), "spam", 0, this, SLOT(slotComplain()), actionCollection(), "complain");
	m_actionStop = new KAction(i18n("S&top current transfer"), "stop", 0, this, SLOT(slotStop()), actionCollection(), "stop");
  m_actionClear = new KAction(i18n("&Clear list"), 0, CTRL+Key_C, this, SLOT(slotClear()), actionCollection(), "clear");

	// setup menu
	m_actionSetupAccounts = new KAction(i18n("Setup &accounts"), "tool", Key_F9, this, SLOT(slotSetupAccounts()), actionCollection(), "setup");
	m_actionSetupAccount = new KAction(i18n("Setup &account"), "tool", 0, this, SLOT(slotSetupAccount()), actionCollection(), "setup2");
	m_actionSetupCommands = new KAction(i18n("Setup &commands"), Key_F10, this, SLOT(slotSetupCommands()), actionCollection(), "commands");
	m_actionSetupFilters = new KAction(i18n("Setup &filters"), "filter", 0, this, SLOT(slotSetupFilters()), actionCollection(), "filters");
	m_actionToolBar = KStdAction::showToolbar(this, SLOT(slotToggleToolBar()), actionCollection());
	m_actionStatusBar = KStdAction::showStatusbar(this, SLOT(slotToggleStatusBar()), actionCollection());

	m_actionHelp = KStdAction::help (this, SLOT(appHelpActivated()), actionCollection());
  m_actionSendFeedbackMail = new KAction( i18n( "Send &Feedback Mail" ), "email",
					 this, SLOT(slotSendFeedbackMail() ),actionCollection());
}

void KShowMailApp::initMenuBar()
{
  m_pFileMenu 	= new QPopupMenu();
  m_pSetupMenu 	= new QPopupMenu();
  m_pActionMenu = new QPopupMenu();
  m_pHelpMenu		= new KHelpMenu(this, m_pAbout);

  m_actionSave->plug (m_pFileMenu);
  m_actionQuit->plug (m_pFileMenu);

  m_actionSetupAccounts->plug (m_pSetupMenu);
  m_actionSetupCommands->plug (m_pSetupMenu);
  m_actionSetupFilters->plug (m_pSetupMenu);
	m_pSetupMenu->insertSeparator ();
	m_actionToolBar->plug (m_pSetupMenu);
	m_actionStatusBar->plug (m_pSetupMenu);
  m_pHelpMenu->menu()->insertSeparator ();
  m_actionSendFeedbackMail->plug (m_pHelpMenu->menu());

//  is called separately by readOptions
//	resetActionMenu ();

	menuBar()->insertItem (i18n ("&File"), m_pFileMenu);
	menuBar()->insertItem (i18n ("&Setup"), m_pSetupMenu);
	menuBar()->insertItem (i18n ("&Actions"), m_pActionMenu);
  menuBar()->insertSeparator();
  menuBar()->insertSeparator();
  menuBar()->insertItem (i18n ("&Help"), m_pHelpMenu->menu());

	// this is for user commands. Should be KAction too, how to ?
  connect(m_pActionMenu, SIGNAL(activated(int)), SLOT(slotCommand(int)));
}

void KShowMailApp::initToolBar()
{
  m_actionSave->plug (toolBar ());
	m_actionRefresh->plug (toolBar ());
	m_actionShowHeader->plug (toolBar ());
	m_actionShowMessage->plug (toolBar ());
	m_actionDelete->plug (toolBar ());
	m_actionComplain->plug (toolBar ());
  toolBar()->insertSeparator();
  m_actionSetupFilters->plug (toolBar ());
	m_actionSetupAccounts->plug (toolBar ());
  m_actionStop->plug (toolBar ());
  toolBar()->insertSeparator();
	m_actionQuit->plug (toolBar ());
}

void KShowMailApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
  QCString user = getenv( "LOGNAME" );
  QCString server = getenv( "HOSTNAME" );
  QCString  mesg (QString ("%1@%2").arg(user).arg(server));

	statusBar()->insertItem(i18n("Ready."), 1);
  statusBar()->insertItem(mesg, 2, 1);
  statusBar()->insertItem("", 3);
  statusBar()->insertItem("", 4);
}

void KShowMailApp::initDocument()
{
  m_pDoc = new KshowmailDoc(this);
  m_pDoc->newDocument();
}

void KShowMailApp::initView()
{ 
  ////////////////////////////////////////////////////////////////////
  // create the main widget here that is managed by KMainWindow's view-region and
  // connect the widget to your document to display document contents.

  m_pView = new KshowmailView(this);
  m_pDoc->addView(m_pView);
  setCentralWidget(m_pView);
}

void KShowMailApp::slotSaveOptions()
{
  kdDebug() << "KShowMailApp::saveOptions()" << endl;
  
  KConfig* config = KGlobal::config();

  // save status of KMessageBox dont show again
  config->setGroup ("Notification Messages");
  QString autofilteraskagain = config->readEntry ("autofilteraskagain");
  QString savepasswdaskagain = config->readEntry ("savepasswdaskagain");

  // delete old configfile to make sure deleted accounts stay deleted
	QString configfilename = locateLocal ("config", "kshowmailrc");
	QString backupfilename = locateLocal ("config", "kshowmailrc.save");
	rename (configfilename, backupfilename);

  // restore status of KMessageBox dont show again
  config->setGroup ("Notification Messages");
  config->writeEntry ("autofilteraskagain", autofilteraskagain);
  config->writeEntry ("savepasswdaskagain", savepasswdaskagain);
  
  config->setGroup("General Options");
  config->writeEntry("Geometry", size());
  config->writeEntry("showtoolbar", m_actionToolBar->isChecked());
  config->writeEntry("showstatusbar",m_actionStatusBar->isChecked());
  config->writeEntry("ToolBarPos", (int) toolBar()->barPos());
  config->writeEntry("ToolBarIconText", (int) toolBar()->iconText());
  config->writeEntry("ToolBarIconSize", toolBar()->iconSize());

  config->setGroup("Mail Options");
	
	m_CommandList.saveOptions (config);
	m_pView->saveOptions (config);
  Filter::saveOptions (config);

	// this should be called by the destructor of KConfig. However, if we don't do it,
	// the configuration is not always saved.
	config->sync ();

	// configlist uses XML file
	m_ConfigList.saveOptions ();
	
//	kdDebug() << "KShowMailApp::saveOptions() end" << endl;
}

void KShowMailApp::readOptions()
{
  KConfig* config = KGlobal::config();
	QString configfilename = locateLocal ("config", "kshowmailrc");
	struct stat filestatus;
	int ok = stat (configfilename, &filestatus);
	if ((ok != 0) || (filestatus.st_size == 0))
	{
		// this happens for unknown reasons. Try to copy backup
		kdDebug() << "Config file corrupt. Try to restore backup." << endl;
		QString backupfilename = locateLocal ("config", "kshowmailrc.save");
	  ok = stat (backupfilename, &filestatus);
    if (ok && (filestatus.st_size > 0))
  	{
  		KRun::runCommand ("cp " + backupfilename + " " + configfilename);
	  	config->reparseConfiguration ();
  	}
	}
		
  config->setGroup("General Options");

  // bar status settings
	m_actionToolBar->setChecked (config->readBoolEntry ("showtoolbar", true));
	m_actionStatusBar->setChecked (config->readBoolEntry ("showstatusbar", true));
  slotToggleToolBar ();
  slotToggleStatusBar ();

  // bar position settings
  toolBar()->setBarPos((KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top));
  toolBar()->setIconText((KToolBar::IconText) config->readNumEntry("ToolBarIconText", KToolBar::IconOnly));
  toolBar()->setIconSize(config->readNumEntry("ToolBarIconSize", 0));
	
  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    resize(size);
  }

	m_ConfigList.readOptions (m_pView->m_pListAccounts);
	m_CommandList.readOptions (config);
  resetActionMenu ();
	
	m_pView->readOptions (config);
  Filter::readOptions (config);
}

bool KShowMailApp::queryClose()
{
  kdDebug() << "KShowMailApp::queryClose()" << endl;
  if (m_ConfigList.m_bConfirmClose)
  {
    bool result = (KMessageBox::questionYesNo (this, i18n ("KShowmail will be closed.\n"
                                      "Are you sure?")) == KMessageBox::Yes);
    return result;
  }
  else
    return true;
}

bool KShowMailApp::queryExit()
{
  kdDebug() << "KShowMailApp::queryExit()" << endl;
  slotSaveOptions ();
  return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KShowMailApp::slotFileQuit()
{
  kdDebug () << "KShowMailApp::slotFileQuit()" << endl;
  if(memberList)
  {
    KMainWindow* w = memberList->first();
    w->close();
  }
}

void KShowMailApp::slotSetupAccounts()
{
	if (m_state != idle)
  {
    kapp->beep();
		return;
  }
	m_state = configure;
	OptionDialog dlg (this, "Options", &m_ConfigList, &m_CommandList);
	if (dlg.exec () == QDialog::Accepted)
	{  
    m_ConfigList.setList (m_pView->m_pListAccounts);
    m_pView->showOrHideColumns(&m_ConfigList); //added by AM
		restartTimer ();
	}
	m_state = idle;
}

void KShowMailApp::slotSetupAccount()
{
	ConfigElem* pConfig = m_ConfigList.current ();
	if (pConfig)
	{
		ServerDialog dlg (this, "Server Options", pConfig, NULL);
		dlg.exec ();
	}
}

void KShowMailApp::slotAccountActivated (QListViewItem* pItem)
{
	QPtrListIterator<ConfigElem> it(m_ConfigList);
	// this is = not == !!!
	while (ConfigElem* pConfig = it())
	{
		if (pConfig->m_pViewItem == pItem)
		{
			setCaption (pConfig->m_strAccount);
			return;
		}
	}
}

void KShowMailApp::slotAlertDestroyed ()
{
  m_pAlertDialog = NULL;
}

void KShowMailApp::slotAlertOk ()
{
//	kdDebug() << "KShowMailApp::slotAlertOk" << endl;
  showNormal ();
	raise ();
}

void KShowMailApp::restartTimer ()
{
	if (m_ConfigList.m_nInitTimer > 0)
	{
		m_pTimer->start (m_ConfigList.m_nInitTimer * 1000, true);
		m_nSecondsToGo = m_ConfigList.m_nInitTimer;
	}
	else if (m_ConfigList.m_nIntervalTimer > 0)
	{
		m_pTimer->start (m_ConfigList.m_nIntervalTimer * 60000, true);
		m_nSecondsToGo = m_ConfigList.m_nIntervalTimer * 60;
	}
	else
	{
		m_pTimer->stop ();
		m_nSecondsToGo = 0;
	}
}

void KShowMailApp::slotRefresh()
{
	if (m_state != idle)
  {
    kapp->beep ();
		return;
  }

	kdDebug() << "slotRefresh" << endl;
	if (m_pTimer->isActive())
	{
		m_pTimer->stop ();
		m_nSecondsToGo = 0;
	}
  if (m_ConfigList.m_nIntervalTimer > 0)
	{
		m_pTimer->start (m_ConfigList.m_nIntervalTimer * 60000, true);
		m_nSecondsToGo = m_ConfigList.m_nIntervalTimer * 60;
	}
	
	// are there active accounts ?
	ConfigElem* pConfig = m_ConfigList.first();
	while (pConfig && !pConfig->m_bActive)
		pConfig = m_ConfigList.next ();
	if (pConfig == NULL)
		return;
  pConfig->first();

  m_nNewMail = 0;
  m_nDeletedMail = 0;
  m_nTotalSize = 0;
  m_nTotalCount = 0;

	startGetUIDLs ();
}

void KShowMailApp::slotForcedRefresh ()
{
  slotClear ();
  slotRefresh ();
}

void KShowMailApp::startGetIndex ()
{
	kdDebug() << "KShowMailApp::startGetIndex" << endl;

	ConfigElem* pConfig = m_ConfigList.current ();
	slotStatusMsg(i18n ("Logging in to server %1 ...").arg(pConfig->m_url.host()));

  pConfig->m_url.setPath ("/index");
  m_pPop3 = KIO::get (pConfig->m_url.url(), false, false);
  m_state = waitIndexResult;
  m_data = "";
  connect (m_pPop3, SIGNAL (data( KIO::Job*, const QByteArray &)), SLOT( slotData( KIO::Job*, const QByteArray &)));
  connect (m_pPop3, SIGNAL (result(KIO::Job*)), SLOT (slotIndexResult(KIO::Job*)));
  connect (m_pPop3, SIGNAL (connected(KIO::Job*)), SLOT (slotConnected(KIO::Job*)));
	m_pPop3Timer->start (m_ConfigList.m_nPop3Timer * 1000, true);
}

void KShowMailApp::slotData (KIO::Job* pop3, const QByteArray& data)
{	
//	kdDebug() << "slotData: " << QString (data) << endl;
	m_data += '\n' + QString (data);
	if (m_state == waitDownloadResult)
	{
		if (m_data.length() >= (m_ConfigList.m_nMaxMailSize*1024))
		{
			kdDebug() << "Download truncated" << endl;
			m_pPop3->kill ();
			slotDownloadResult (pop3);
		}
	}
}

void KShowMailApp::slotConnected (KIO::Job*)
{
	ConfigElem* pConfig = m_ConfigList.current();
	if (pConfig == NULL)
		return;

	kdDebug() << i18n("connection to pop3 server %1 established successfully").arg(pConfig->m_url.host ()) << endl;
	slotStatusMsg (i18n("connection to pop3 server %1 established successfully").arg(pConfig->m_url.host ()));
}

void KShowMailApp::slotStop ()
{
	if (m_state == idle)
  {
    kapp->beep ();
    return;
  }
	kdDebug() << "KShowMailApp::slotStop" << endl;
	m_pPop3Timer->stop ();
	if (m_pPop3)
		m_pPop3->kill ();

  slotRefreshView ();
  // make sure the view is refreshed before the state gets idle again
  // bug #853651
	m_state = idle;
	// might be more than one override cursor ...
	while (QApplication::overrideCursor())
		QApplication::restoreOverrideCursor ();
	slotStatusMsg(i18n("Job was stopped"));
}

void KShowMailApp::slotClear ()
{
	if (m_state != idle)
  {
    kapp->beep ();
		return;
  }
	kdDebug() << "KShowMailApp::slotClear" << endl;

	for (ConfigElem* pConfig = m_ConfigList.first(); pConfig; pConfig = m_ConfigList.next())
	{
		if (!pConfig->m_bActive)
			continue;
		pConfig->m_pshowrecord->clear();
//		pConfig->m_pViewItem->setText (4, "?");
//		pConfig->m_pViewItem->setText (5, "?");
	}
//	m_nTotalCount = 0;

	slotRefreshView ();
}

void KShowMailApp::slotPop3Timeout ()
{
	kdDebug() << "KShowMailApp::slotPop3Timeout: state = ";
	m_pPop3->kill ();
  deleteIncompleteRecords ();
  if (m_pMessagebox == NULL)
    m_pMessagebox = new QMessageBox (0);
  m_pMessagebox->setCaption (m_ConfigList.current()->m_strAccount);
  m_pMessagebox->setText (i18n("POP3 Timeout"));
  m_pMessagebox->setIcon (QMessageBox::Critical);
  m_pMessagebox->show ();
  m_pMessagebox->raise ();
	switch (m_state)
	{
		case waitUIDLsResult:
			kdDebug() << "waitUIDLsResult" << endl;
			UIDLsCont ();
			break;
		case waitUIDLsCont:
			kdDebug() << "waitUIDLsCont" << endl;
			UIDLsCont ();
			break;
		case waitIndexResult:
			kdDebug() << "waitIndexResult" << endl;
			IndexCont ();
			break;
		case waitIndexCont:
			kdDebug() << "waitIndexCont" << endl;
			IndexCont ();
			break;
		case waitHeadersResult:
			kdDebug() << "waitHeadersResult" << endl;
	    HeadersCont ();
	    break;
		case waitHeadersCont:
			kdDebug() << "waitHeadersCont" << endl;
			HeadersCont ();
			break;
		case waitDeleteResult:
			kdDebug() << "waitDeleteResult" << endl;
			DeleteCont ();
			break;
		case waitDeleteCont:
			kdDebug() << "waitDeleteCont" << endl;
			DeleteCont ();
			break;
		case waitComplainResult:
			kdDebug() << "waitComplainResult" << endl;
			ComplainCont ();
			break;
		case waitComplainCont:
			kdDebug() << "waitComplainCont" << endl;
			ComplainCont ();
			break;
		case waitDownloadResult:
			kdDebug() << "waitDownloadResult" << endl;
			DownloadCont ();
			break;
		case waitDownloadCont:
			kdDebug() << "waitDownloadCont" << endl;
			DownloadCont ();
			break;
		default:
			kdDebug() << "other" << endl;
			m_state = idle;
    	while (QApplication::overrideCursor())
      	QApplication::restoreOverrideCursor ();
    	slotStatusMsg(i18n("Timeout: job was killed"));
	}
}

void KShowMailApp::deleteIncompleteRecords ()
{
	ConfigElem* pConfig = m_ConfigList.current();

	for (QListIterator<ShowRecordElem>it(*pConfig->m_pshowrecord); it.current(); ++it)
	{
		if (it.current()->from () == "???")
		{
		  it.current()->setDeleted ();
    }
  }
}

void KShowMailApp::scanList ()
{
 	// m_data contains a newline separated list of the form:
 	// 1 1234
 	// 2 2345
 	// 3 size etc
 	kdDebug() << "KShowMailApp::scanList" << endl;
 	int pos1 = 0;
 	int pos2 = m_data.find ('\n', 1);
 	ConfigElem* pConfig = m_ConfigList.current();
 	pConfig->first();
 	while (pos2 >= 0)
 	{
 	  int pos3 = m_data.find (' ', pos1, pos2 - pos1);
 	  bool ok;
 	  int size = m_data.mid (pos3+1, pos2 - pos3 - 1).toInt (&ok);
 	  if (ok)
 	  {
 	  	while (pConfig->current() && pConfig->current()->deleted())
 	  		pConfig->next();
 	  	// this should never happen
 	  	if (pConfig->current() == NULL)
 	  		break;
//		kdDebug() << "Number = " << pConfig->current()->number() << "; size = " << size << endl;
			pConfig->current()->setSize (size);
 	    m_nTotalSize += size;
  		pConfig->next();
 	  }
 	  else
 	    break;
 	  pos1 = pos2 + 1;
 	  pos2 = m_data.find ('\n', pos1);
 	}
}

void KShowMailApp::scanUIDLs ()
{
 	// m_data contains a newline separated list of the form:
 	// 1 uidlstring1
 	// 2 uidlstring2
 	// 3 uidlstring3
 	ConfigElem* pConfig = m_ConfigList.current();
 	kdDebug() << "KShowMailApp::scanUIDLs: " << pConfig->m_strAccount << endl;
 	pConfig->first();
 	int pos1 = 0;
 	int pos2 = m_data.find ('\n', 1);
 	int nCounter = 0;
 	while (pos2 >= 0)
 	{
 	  int pos3 = m_data.find (' ', pos1, pos2 - pos1);
 	  QString uidl = m_data.mid (pos3+1, pos2 - pos3 - 1);
 	
 	  if (pConfig->current())
 	  {
 	  	// existing entry
 	  	if (pConfig->current()->uidl() == uidl)
 	  	{
 	  		// old message
// 	  		kdDebug() << "Old msg, UIDL OK: " << nCounter+1 << endl;
 	  		if (!m_ConfigList.m_bKeepNew)
 	  			pConfig->current()->setNew (false);
		    // repair number, just in case
		    pConfig->current()->setNumber (nCounter+1);
		    pConfig->next();
		    nCounter++;
 	  	}
 	  	else
 	  	{
 	  		// old message, wrong uidl; message deleted by other application
 	  		kdDebug() << "Old msg, wrong UIDL:" << nCounter+1 << endl;
 	  		pConfig->current()->setNew (false);
 	  		pConfig->current()->setDeleted ();
		 		m_nDeletedMail++;
				// try next uidl; if no one matches, all remaining messages will be treated as new
		    pConfig->next();
				continue;
 	  	}
 	  }
 	  else
 	  {
 	  	// new message
 	 		kdDebug() << "New msg: " << nCounter+1 << endl;
 	 		pConfig->append (new ShowRecordElem ());
 	 		// the new item becomes current
 	 		pConfig->current()->setNew (true);
	    pConfig->current()->setUIDL (uidl);
	    pConfig->current()->setNumber (nCounter+1);
	    // this will set current to NULL
	    pConfig->next();
	 	  nCounter++;
  		m_nNewMail++;
 	  }
 	  pos1 = pos2 + 1;
 	  pos2 = m_data.find ('\n', pos1);
 	}
 	m_nTotalCount += nCounter;
 	m_nMessages = nCounter;
 	while (pConfig->current())
 	{
 		// these messages are still in our list, but not on the server; delete them too
 		kdDebug() << "setDeleted: " << nCounter << endl;
 		pConfig->current()->setDeleted ();
 		pConfig->current()->setNew (false);
 		m_nDeletedMail++;
 	  nCounter++;
 	  pConfig->next();
 	}
}

void KShowMailApp::slotRefreshView ()
{
	kdDebug() << "KShowMailApp::refreshView" << endl;
	m_nTotalSize = 0;
	m_nTotalCount = 0;
	ShowListViewItem* last = NULL;
	ShowListViewItem* pFirstNewItem = NULL;
	m_pView->m_pListMessages->clear();
  int oldcol = m_pView->m_pListMessages->columnSorted ();
  bool oldascend = m_pView->m_pListMessages->ascendingSort ();
  // sort later
  m_pView->m_pListMessages->setSorting(-1);
	for (ConfigElem* pConfig = m_ConfigList.first(); pConfig; pConfig = m_ConfigList.next())
	{
		if (!pConfig->m_bActive)
			continue;
		int nCounter = 0;
		int nAccountSize = 0;
//		kdDebug() << "account = " << pConfig->m_strAccount << "/" << pConfig->count() << endl;
		for (QListIterator<ShowRecordElem>it(*pConfig->m_pshowrecord); it.current();)
		{
			if (it.current()->deleted ())
			{
				kdDebug() << "removing number: " << it.current()->number () << endl;
				if (it.atLast())
				{
					pConfig->removeRef (it.current());
					break;
				}
				else
        {
					pConfig->removeRef (it.current());
				  continue;
        }
			}
			else
			{
				nCounter++;
				it.current()->setNumber (nCounter);
				m_nTotalSize += it.current()->size();
				nAccountSize += it.current()->size();
			}
			
//			kdDebug() << "new ShowListViewItem: " << it.current()->number() << endl;
			last = new ShowListViewItem (m_pView->m_pListMessages, last);
      last->setNumber (QString ("%1").arg(it.current()->number(),4));
      last->setAccount(pConfig->m_strAccount);
      last->setFrom(it.current()->from());
      last->setTo(it.current()->to());
      last->setSubject(it.current()->subject());
      last->setDate(it.current()->date());
      last->setSize(it.current()->strSize());
      last->setContent(it.current()->content());
      last->setState(it.current()->state());
			last->setTime (it.current()->strUnixTime());
			it.current()->setViewItem (last);
			if ((pFirstNewItem == NULL) && it.current()->isNew())
				pFirstNewItem = last;
			++it;
		}
    // set counter and size in account list view
		pConfig->m_pViewItem->setText (4, QString ("%1").arg (nCounter, 3));
		pConfig->m_pViewItem->setText (5, QString ("%1").arg (nAccountSize, 8));
		m_nTotalCount += nCounter;
	}
  // now do the sorting
  m_pView->m_pListMessages->setSorting(oldcol, oldascend);
  if (last)
    m_pView->m_pListMessages->ensureItemVisible (last);
  if (pFirstNewItem)
    m_pView->m_pListMessages->ensureItemVisible (pFirstNewItem);
	if (m_nNewMail > 0 || m_nDeletedMail > 0)
 		m_pDockWindow->drawNumber (m_nTotalCount, Qt::red);
	else
		m_pDockWindow->drawNumber (m_nTotalCount, Qt::black);
  m_ConfigList.applyFilters ();
	
	slotStatusMsg(i18n("%1 message(s) with a total of %2 bytes are waiting").arg(m_nTotalCount).arg(m_nTotalSize));           							
}

void KShowMailApp::slotIndexResult (KIO::Job* job)
{
	kdDebug() << "KShowMailApp::slotIndexResult" << endl;
	m_pPop3Timer->stop ();
  if (job->error())
  {
    kdDebug() << "KShowMailApp::slotIndexResult: KIO::error" << endl;
    job->showErrorDialog (this);
  }
  	
	ConfigElem* pConfig = m_ConfigList.current();
	if (pConfig == NULL)
		return;
 	scanList ();
	
	QString strCounter;
	strCounter = QString ("%1").arg (m_nMessages);
	pConfig->m_pViewItem->setText (4, strCounter);
  	
	QApplication::restoreOverrideCursor ();
	if (m_nMessages > 0)
	{
		pConfig->first();
		startGetHeaders ();
	}
	else
	{
 		// no headers, terminate current session
 		pConfig->m_url.setPath ("/commit");
 		m_pPop3 = KIO::get (pConfig->m_url.url(), false, false);
 		m_state = waitIndexCont;
		QApplication::restoreOverrideCursor ();
		connect (m_pPop3, SIGNAL (result(KIO::Job*)), SLOT (slotIndexCont(KIO::Job*)));
		m_pPop3Timer->start (m_ConfigList.m_nPop3Timer * 1000, true);
	}
}

void KShowMailApp::slotUIDLsResult (KIO::Job* job)
{
	kdDebug() << "KShowMailApp::slotUIDLsResult" << endl;
 	ConfigElem* pConfig = m_ConfigList.current();
	m_pPop3Timer->stop ();
  int error_code = job->error();
  if (error_code)
  {
    kdDebug() << "KShowMailApp::slotUIDLsResult: KIO::error" << endl;
    kdDebug() << "error_code: " << error_code << endl;
    kdDebug() << job->errorString() << endl;
    job->showErrorDialog (this);
    switch (error_code)
    {
      case ERR_COULD_NOT_LOGIN:
      case ERR_COULD_NOT_AUTHENTICATE:
      case ERR_NEED_PASSWD:
        assertPasswd (pConfig, true);
        break;
      case ERR_SLAVE_DIED:
        m_state = idle;
        while (QApplication::overrideCursor())
          QApplication::restoreOverrideCursor ();
        slotStatusMsg(i18n("Timeout: job was killed"));
        return;
      default:
        ;
    }
    UIDLsCont ();
    return;
  }

 	scanUIDLs ();
 	if (m_nMessages > 0)
 	{
    pConfig->first();
		startGetIndex ();
	}
	else
	{
 		// no headers, terminate current session
 		pConfig->m_url.setPath ("/commit");
 		m_pPop3 = KIO::get (pConfig->m_url.url(), false, false);
 		m_state = waitUIDLsCont;
		QApplication::restoreOverrideCursor ();
		connect (m_pPop3, SIGNAL (result(KIO::Job*)), SLOT (slotUIDLsCont(KIO::Job*)));
		m_pPop3Timer->start (m_ConfigList.m_nPop3Timer * 1000, true);
	}
}

void KShowMailApp::slotIndexCont (KIO::Job* job)
{
	m_pPop3Timer->stop();
  if (job->error())
  {
    kdDebug() << "KShowMailApp::slotIndexCont: KIO::error" << endl;
    job->showErrorDialog (this);
  }

  IndexCont();
}

void KShowMailApp::slotUIDLsCont (KIO::Job* job)
{
	m_pPop3Timer->stop();
  if (job->error())
  {
    kdDebug() << "KShowMailApp::slotUIDLsCont: KIO::error" << endl;
    job->showErrorDialog (this);
  }

  UIDLsCont();
}

void KShowMailApp::finalizeRefresh ()
{
	kdDebug() << "KShowMailApp::finalizeRefresh" << endl;
	slotRefreshView ();

  // make sure the view is refreshed before the state gets idle again
  // bug #853651
	m_state = idle;
	while (QApplication::overrideCursor())
  	QApplication::restoreOverrideCursor();

  if (m_nNewMail > 0)
	{
		kdDebug() << "new mail was found" << endl;
		if (m_ConfigList.m_bShowMainWindow)
		{
//			kdDebug() << "show main window" << endl;
			showNormal ();
      raise ();
		}
		m_ConfigList.beep ();
		m_ConfigList.playSound ();
		if (m_ConfigList.m_bShowMessage)
		{
			if (m_pAlertDialog)
			{
				m_pAlertDialog->show ();
				m_pAlertDialog->raise ();
			}
			else
			{
//				kdDebug() << "new AlertDialog" << endl;
				m_pAlertDialog = new AlertDialog (this);
				connect (m_pAlertDialog, SIGNAL (destroyed()), SLOT (slotAlertDestroyed()));
				connect (m_pAlertDialog, SIGNAL (signalOk()), SLOT (slotAlertOk()));
			}
		}
//		m_ConfigList.startProgram (&m_CommandList, pConfig);
		m_ConfigList.startProgram (&m_CommandList, NULL);
	}
  else
	{
		if (m_ConfigList.m_bTerminate)
			slotFileQuit ();
		else if (m_ConfigList.m_bMinimize)
    {
			showMinimized ();
      hide ();
    }
	}
  if (Filter::_status == Filter::automatic)
  {
    slotDelete ();
  }
}

void KShowMailApp::finalizeDelete ()
{
	kdDebug() << "KShowMailApp::finalizeDelete" << endl;
  // make sure the view is refreshed before the state gets idle again
  // bug #853651
  slotRefreshView ();

  m_state = idle;
	while (QApplication::overrideCursor())
  	QApplication::restoreOverrideCursor();
}

void KShowMailApp::IndexCont ()
{
	kdDebug() << "KShowMailApp::IndexCont" << endl;
	ConfigElem* pConfig = m_ConfigList.next ();
	while (pConfig && !pConfig->m_bActive)
		pConfig = m_ConfigList.next ();
	if (pConfig)
		startGetIndex ();
	else
		finalizeRefresh ();
}

void KShowMailApp::UIDLsCont ()
{
	kdDebug() << "KShowMailApp::UIDLsCont" << endl;
	ConfigElem* pConfig = m_ConfigList.next ();
	while (pConfig && !pConfig->m_bActive)
		pConfig = m_ConfigList.next ();
	if (pConfig)
		startGetUIDLs ();
	else
		finalizeRefresh ();
}

bool KShowMailApp::assertPasswd (ConfigElem* pConfig, bool force)
{
  if (pConfig->m_url.pass().isEmpty() || force)
	{
    KPasswordDialog dlg (KPasswordDialog::Password, i18n("Password"), this);
    dlg.raise();
    int result = dlg.exec();
    if (result == KPasswordDialog::Accepted)
		{
      QCString password = dlg.password();
			pConfig->m_url.setPass (password);
			return true;
		}
		else
			return false;
	}
	else
		return true;
}	

void KShowMailApp::startGetUIDLs ()
{
	ConfigElem* pConfig = m_ConfigList.current();
	kdDebug() << "KShowMailApp::startGetUIDLs: " << pConfig->m_strAccount << endl;

	if (!assertPasswd (pConfig))
		return;
		
	QApplication::setOverrideCursor (Qt::waitCursor);
	pConfig->m_url.setPath (QString ("/uidl"));
 	m_pPop3 = KIO::get (pConfig->m_url.url(), false, false);
 	m_state = waitUIDLsResult;
	
	m_data = "";
	connect (m_pPop3, SIGNAL (data( KIO::Job*, const QByteArray &)), SLOT(slotData(KIO::Job*, const QByteArray &)));
	connect (m_pPop3, SIGNAL (result(KIO::Job*)), SLOT (slotUIDLsResult(KIO::Job*)));
	m_pPop3Timer->start (m_ConfigList.m_nPop3Timer * 1000, true);

	slotStatusMsg(i18n("getting UIDLs ..."));
}

void KShowMailApp::startGetHeaders ()
{
	ConfigElem* pConfig = m_ConfigList.current();
	kdDebug() << "KShowMailApp::startGetHeaders: " << pConfig->m_strAccount << endl;
	
	while (pConfig &&
	       pConfig->current() &&
	       !pConfig->current()->isNew())
		pConfig->next();
	if (pConfig->current() == NULL)
	{
 		kdDebug() << "no more headers, terminate current session" << endl;
 		pConfig->m_url.setPath ("/commit");
 		m_pPop3 = KIO::get (pConfig->m_url.url(), false, false);
//		QApplication::restoreOverrideCursor ();
		m_state = waitHeadersCont;
		connect (m_pPop3, SIGNAL (result(KIO::Job*)), SLOT (slotHeadersCont(KIO::Job*)));
		m_pPop3Timer->start (m_ConfigList.m_nPop3Timer * 1000, true);
	}
	else
	{
		pConfig->m_url.setPath (QString ("/headers/%1").arg (pConfig->current()->number()));
 		m_pPop3 = KIO::get (pConfig->m_url.url(), false, false);
 		m_state = waitHeadersResult;
		m_data = "";
		connect (m_pPop3, SIGNAL (data( KIO::Job*, const QByteArray &)), SLOT(slotData(KIO::Job*, const QByteArray &)));
		connect (m_pPop3, SIGNAL (result(KIO::Job*)), SLOT (slotHeadersResult(KIO::Job*)));
		m_pPop3Timer->start (m_ConfigList.m_nPop3Timer * 1000, true);

		slotStatusMsg(i18n("getting message %1 of %2 ...").arg (pConfig->current()->number()).arg (m_nMessages));
	}
}

void KShowMailApp::slotHeadersResult (KIO::Job* job)
{
	ConfigElem* pConfig = m_ConfigList.current();
  kdDebug() << "KShowMailApp::slotHeadersResult: " << pConfig->m_strAccount << endl;
	m_pPop3Timer->stop();
  if (job->error())
  {
    kdDebug() << "KShowMailApp::slotHeadersResult: KIO::error" << endl;
    job->showErrorDialog (this);
  }
  	
	if (pConfig == NULL)
		return;
	
	QString number (QString ("%1").arg (pConfig->current()->number(),4));
	
	pConfig->current()->setHeader (m_data);
		
	pConfig->next();	
	// check for more
	if (pConfig->current())
	{
		startGetHeaders ();
	}
	else
	{
 		// no more headers, terminate current session
 		kdDebug() << "no more headers, terminate current session" << endl;
 		pConfig->m_url.setPath ("/commit");
 		m_pPop3 = KIO::get (pConfig->m_url.url(), false, false);
		QApplication::restoreOverrideCursor ();
		m_state = waitHeadersCont;
		connect (m_pPop3, SIGNAL (result(KIO::Job*)), SLOT (slotHeadersCont(KIO::Job*)));
		m_pPop3Timer->start (m_ConfigList.m_nPop3Timer * 1000, true);
	}
}		
		
void KShowMailApp::slotHeadersCont (KIO::Job* job)
{
	m_pPop3Timer->stop();
  if (job->error())
  {
    kdDebug() << "KShowMailApp::slotHeadersCont: KIO::error" << endl;
    job->showErrorDialog (this);
  }
  HeadersCont ();
}
  		
void KShowMailApp::HeadersCont ()
{
 	// find next active account
 	ConfigElem* pConfig = m_ConfigList.next ();
 	while (pConfig && !pConfig->m_bActive)
  	pConfig = m_ConfigList.next ();
 	if (pConfig)
 	{
 		pConfig->first();
		startGetUIDLs ();
	}
 	else
 	{
 		// no more active accounts
 		finalizeRefresh ();
	}
}

void KShowMailApp::slotDelete()
{
	if ((m_state != idle) || (m_ConfigList.first() == NULL))
  {
    kapp->beep ();
		return;
  }
	
	m_bYesAll = false;
	m_bNoAll = false;
	m_ConfigList.first()->first();
	m_ConfigList.skipHighLight ();
	if (m_ConfigList.current())
	{
		QApplication::setOverrideCursor (Qt::waitCursor);
		m_state = deleting;
		slotStatusMsg(i18n("Deleting highlighted messages..."));
		startDelete ();
  }
}

void KShowMailApp::startDelete ()
{
  kdDebug () << "KShowMailApp::startDelete" << endl;
	bool bDelete = false;
	if (m_bYesAll)
	{
		bDelete = true;
	}
	else if (m_bNoAll)
	{
		bDelete = false;
	}
	else if (m_ConfigList.m_bConfirmDelete)
	{
		QApplication::setOverrideCursor (Qt::arrowCursor);
		
		int number = m_ConfigList.current()->current()->number ();
		QString account = m_ConfigList.current()->m_strAccount;
		
		int flag = QMessageBox::warning( this, i18n("Warning"),
			i18n("Message %1 at %2 will be deleted.\nAre you sure ?\n").arg(number).arg (account),
			i18n("Yes"), i18n ("Yes all"), i18n("No"), 2);
		QApplication::restoreOverrideCursor ();
		switch (flag)
		{
			case 0: // yes
				bDelete = true;
				break;
			case 1: // yes all
				bDelete = true;
				m_bYesAll = true;
				break;
			case 2: // no
				bDelete = false;
				break;
			case -1: // cancel, no all
				bDelete = false;
				m_bNoAll = true;
		}
  }
	else
		bDelete = true;
		
	if (bDelete)
	{
	 	ConfigElem* pConfig = m_ConfigList.current();
	 	kdDebug() << "removing: " << pConfig->current()->number() << " from " << pConfig->current()->from() << endl;
	 	
	 	assertPasswd (pConfig);
	 	
	 	pConfig->m_url.setPath (QString ("/remove/%1").arg (pConfig->current()->number()));
		m_pPop3 = KIO::get (pConfig->m_url.url(), false, false);
		m_state = waitDeleteResult;
		m_data = "";
		connect (m_pPop3, SIGNAL (data( KIO::Job*, const QByteArray &)), SLOT( slotData( KIO::Job*, const QByteArray &)));
		connect (m_pPop3, SIGNAL (result(KIO::Job*)), SLOT (slotDeleteResult(KIO::Job*)));
		m_pPop3Timer->start (m_ConfigList.m_nPop3Timer * 1000, true);
	}
	else
	{
		m_ConfigList.nextHighLight ();
		if (m_ConfigList.current())
			startDelete ();
		else
			finalizeDelete ();
	}
}

/**
  * slotDeleteResult will continue the delete process after a remove call
  * this is done for each message to be deleted
  */
void KShowMailApp::slotDeleteResult (KIO::Job* job)
{
	m_pPop3Timer->stop ();
  if (job->error())
  {
    kdDebug() << "KShowMailApp::slotDeleteResult: KIO::error" << endl;
    job->showErrorDialog (this);
  }
	
	m_ConfigList.current()->current()->setDeleted ();
	// delete item from listview, to get a more responsive look and feel
	delete m_ConfigList.current ()->current()->viewItem ();

	ConfigElem* pLastElem = m_ConfigList.current();
	m_ConfigList.nextHighLight ();
 	ConfigElem* pConfig = m_ConfigList.current();
 	
 	if (pConfig != pLastElem)
 	{
	 	// if the account has changed, we must close the old one
 		pLastElem->m_url.setPath ("/commit");
 		m_pPop3 = KIO::get (pLastElem->m_url.url(), false, false);
		QApplication::restoreOverrideCursor ();
 		m_state = waitDeleteCont;
		connect (m_pPop3, SIGNAL (result(KIO::Job*)), SLOT (slotDeleteCont(KIO::Job*)));
		m_pPop3Timer->start (m_ConfigList.m_nPop3Timer * 1000, true);
	}
	else
		startDelete ();
}
	
/**
  * slotDeleteCont will continue the delete process after a submit call;
  * this only happens after one account has been processed completely
  */
void KShowMailApp::slotDeleteCont (KIO::Job* job)
{	
	m_pPop3Timer->stop ();
  if (job->error())
  {
    kdDebug() << "KShowMailApp::slotDeleteCont: KIO::error" << endl;
    job->showErrorDialog (this);
  }

  DeleteCont ();
}

void KShowMailApp::DeleteCont ()
{
// here we don't need to call nextHighLight: it has been done in
// slotDeleteResult
//	m_ConfigList.nextHighLight ();
	if (m_ConfigList.current())
		startDelete ();
	else
		finalizeDelete ();
}

void KShowMailApp::slotComplain()
{
	if ((m_state != idle) || (m_ConfigList.first() == NULL))
  {
    kapp->beep ();
    return;
  }
	
	m_bYesAll = false;
	m_bNoAll = false;
	m_ConfigList.first()->first();
	m_ConfigList.skipHighLight ();
	if (m_ConfigList.current())
	{
		m_state = complaining;
		slotStatusMsg(i18n("Sending complain mails..."));
		startComplain ();
  }
}

void KShowMailApp::startComplain ()
{
	bool bComplain = false;
 	ConfigElem* pConfig = m_ConfigList.current();
	if (m_bYesAll)
	{
		bComplain = true;
	}
	else if (m_bNoAll)
	{
		bComplain = false;
	}
	else if (m_ConfigList.m_bConfirmSend)
	{
		QApplication::setOverrideCursor (Qt::arrowCursor);
		int number = pConfig->current()->number ();
		QString account = pConfig->m_strAccount;
		int flag = QMessageBox::warning( this, i18n("Warning"),
			i18n("Complain mail will be send for message %1 at %2.\nAre you sure ?\n").arg(number).arg(account),
			i18n("Yes"), i18n ("Yes all"), i18n("No"), 2);
		QApplication::restoreOverrideCursor ();
		switch (flag)
		{
			case 0: // yes
				bComplain = true;
				break;
			case 1: // yes all
				bComplain = true;
				m_bYesAll = true;
				break;
			case 2: // no
				bComplain = false;
				break;
			case -1: // cancel, no all
				bComplain = false;
				m_bNoAll = true;
		}
  }
	else
		bComplain = true;
	if (bComplain)
	{
	 	assertPasswd (pConfig);
	 	pConfig->m_url.setPath (QString ("/download/%1").arg (pConfig->current()->number ()));
		m_pPop3 = KIO::get (pConfig->m_url.url(), false, false);
		m_state = waitComplainResult;
		m_data = "";
		connect (m_pPop3, SIGNAL (data( KIO::Job*, const QByteArray &)), SLOT( slotData( KIO::Job*, const QByteArray &)));
		connect (m_pPop3, SIGNAL (result(KIO::Job*)), SLOT (slotComplainResult(KIO::Job*)));
		m_pPop3Timer->start (m_ConfigList.m_nPop3Timer * 1000, true);
	}
	else
	{
		m_ConfigList.nextHighLight ();
		if (m_ConfigList.current())
			startComplain ();
		else
		{
			QApplication::restoreOverrideCursor ();
      m_state = idle;
	  	slotStatusMsg(i18n("Ready."));
		}
	}
}

void KShowMailApp::slotComplainResult (KIO::Job* job)
{
	m_pPop3Timer->stop ();
  if (job->error())
  {
    kdDebug() << "KShowMailApp::slotComplainResult: KIO::error" << endl;
    job->showErrorDialog (this);
  }

 	ConfigElem* pConfig = m_ConfigList.current();
	m_CommandList.exec (ID_OPTION_COMPLAIN, pConfig, pConfig->current()->header(), m_data);

 	pConfig->m_url.setPath ("/commit");
 	m_pPop3 = KIO::get (pConfig->m_url.url(), false, false);
	QApplication::restoreOverrideCursor ();
 	m_state = waitComplainCont;
	connect (m_pPop3, SIGNAL (result(KIO::Job*)), SLOT (slotComplainCont(KIO::Job*)));
	m_pPop3Timer->start (m_ConfigList.m_nPop3Timer * 1000, true);
}

void KShowMailApp::slotComplainCont (KIO::Job* job)
{
	m_pPop3Timer->stop ();
  if (job->error())
  {
    kdDebug() << "KShowMailApp::slotComplainCont: KIO::error" << endl;
    job->showErrorDialog (this);
  }
  ComplainCont ();
}

void KShowMailApp::ComplainCont ()
{
	m_ConfigList.nextHighLight ();
	if (m_ConfigList.current())
		startComplain ();
	else
	{
		m_state = idle;
		QApplication::restoreOverrideCursor ();
  	slotStatusMsg(i18n("Ready."));
	}
}

void KShowMailApp::slotShowHeader ()
{
	kdDebug() << "KShowMailApp::slotShowHeader" << endl;
	if ((m_state != idle) || (m_ConfigList.first() == NULL))
  {
    kapp->beep ();
		return;
  }

	m_state = configure;
	m_ConfigList.first()->first();
	for (m_ConfigList.skipHighLight(); m_ConfigList.current(); m_ConfigList.nextHighLight())
  {
    EditDialog dlg (this, i18n("Header %1 at %2").arg (m_ConfigList.current()->current()->number()).arg(m_ConfigList.current()->m_strAccount),
      m_ConfigList.current()->current()->header(), false);
		switch (dlg.exec ())
    {
      case QDialog::Rejected:
        m_state = idle;
        return;
        break;
      case QDialog::Accepted:
        break;
      case 10:
      {
        KURL mail;
        mail.setProtocol( "mailto" );
        mail.setPath( m_ConfigList.current()->current()->from() );
        mail.setQuery( "?subject="	+ KURL::encode_string( "Re: " +  m_ConfigList.current()->current()->subject() ) +
		      "&body="	+ KURL::encode_string( "" ) );

        kapp->invokeMailer( mail );
        break;
      }
    }
  }
  m_state = idle;
}

void KShowMailApp::slotShowMessage ()
{
	if ((m_state != idle) || (m_ConfigList.first() == NULL))
  {
    kapp->beep ();
    return;
  }
	
	m_ConfigList.first()->first();
	m_ConfigList.skipHighLight ();
	if (m_ConfigList.current())
	{
		m_state = configure;
		slotStatusMsg(i18n("Getting complete highlighted messages..."));
		startShowMessage ();
  }
}

void KShowMailApp::startShowMessage ()
{
 	ConfigElem* pConfig = m_ConfigList.current();

	assertPasswd (pConfig);
 	
 	pConfig->m_url.setPath (QString ("/download/%1").arg (pConfig->current()->number()));
	m_pPop3 = KIO::get (pConfig->m_url.url(), false, false);
	m_state = waitDownloadResult;
	m_data = "";
	QApplication::setOverrideCursor (Qt::waitCursor);
	connect (m_pPop3, SIGNAL (data( KIO::Job*, const QByteArray &)), SLOT( slotData( KIO::Job*, const QByteArray &)));
	connect (m_pPop3, SIGNAL (result(KIO::Job*)), SLOT (slotDownloadResult(KIO::Job*)));
	// do we really need a timer here ? Mails can get large ...
	m_pPop3Timer->start (m_ConfigList.m_nPop3Timer * 1000, true);
}

void KShowMailApp::slotDownloadResult (KIO::Job* job)
{
	QApplication::restoreOverrideCursor ();
	m_pPop3Timer->stop ();
 	ConfigElem* pConfig = m_ConfigList.current();
  if (job->error())
  {
    kdDebug() << "KShowMailApp::slotDownloadResult: KIO::error" << endl;
    job->showErrorDialog (this);
  }
  else
  {
    int pos = m_data.find ("\r\n\r\n") + 4;
    QString body (m_data.mid (pos));
    EditDialog dlg (this, i18n("Message %1 at %2").arg (pConfig->current()->number()).arg(pConfig->m_strAccount),
        body, m_ConfigList.m_bAllowHTML);
    dlg.addWidget(i18n("Sender:"), pConfig->current()->from());
    dlg.addWidget(i18n("Date:"), pConfig->current()->date());
    dlg.addWidget(i18n("Size:"), pConfig->current()->strSize());
    dlg.addWidget(i18n("Subject:"), pConfig->current()->subject());
		switch (dlg.exec ())
    {
      case QDialog::Rejected:
        m_ConfigList.last();
        break;
      case QDialog::Accepted:
        break;
      case 10:
      {
        body.insert (0, "> ");
        body.replace ("\r\n", "\r\n> ");
        KURL mail;
        mail.setProtocol( "mailto" );
        mail.setPath( m_ConfigList.current()->current()->from() );
        mail.setQuery( "?subject="	+ KURL::encode_string( "Re: " +  m_ConfigList.current()->current()->subject() ) +
		      "&body="	+ KURL::encode_string( body ) );

        kapp->invokeMailer( mail );
        break;
      }
    }
  }

	pConfig->m_url.setPath ("/commit");
 	m_pPop3 = KIO::get (pConfig->m_url.url(), false, false);
	QApplication::restoreOverrideCursor ();
 	m_state = waitDownloadCont;
	connect (m_pPop3, SIGNAL (result(KIO::Job*)), SLOT (slotDownloadCont(KIO::Job*)));
	m_pPop3Timer->start (m_ConfigList.m_nPop3Timer * 1000, true);
}
	
void KShowMailApp::slotDownloadCont (KIO::Job* job)
{
	m_pPop3Timer->stop ();
  if (job->error())
  {
    kdDebug() << "KShowMailApp::slotDownloadCont: KIO::error" << endl;
    job->showErrorDialog (this);
  }
	
	DownloadCont ();
}

void KShowMailApp::DownloadCont ()
{
  m_ConfigList.nextHighLight ();
	if (m_ConfigList.current())
		startShowMessage ();
	else
	{
	  slotStatusMsg(i18n("Ready."));
		m_state = idle;
	}
}

void KShowMailApp::slotToggleToolBar()
{
  if(!m_actionToolBar->isChecked())
    toolBar()->hide();
  else
    toolBar()->show();
}

void KShowMailApp::slotToggleStatusBar()
{
  if(!m_actionStatusBar->isChecked())
    statusBar()->hide();
  else
    statusBar()->show();
}

void KShowMailApp::slotStatusMsg(const QString& text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
 	// added by Gustavo Zamorano to include time
  QString sTime = QTime::currentTime ().toString ();

  statusBar()->clear();
  statusBar()->changeItem(text, 1);
  statusBar()->changeItem(sTime, 4);
}

void KShowMailApp::slotStatusHelpMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message of whole statusbar temporary (text, msec)
  statusBar()->message(text, 2000);
}

void KShowMailApp::resetActionMenu ()
{
	m_pActionMenu->clear ();
	m_actionRefresh->plug (m_pActionMenu);
	m_actionForcedRefresh->plug (m_pActionMenu);
	m_actionShowHeader->plug (m_pActionMenu);
	m_actionShowMessage->plug (m_pActionMenu);
	m_actionDelete->plug (m_pActionMenu);
	m_actionComplain->plug (m_pActionMenu);
	m_actionComplain->setEnabled (false);
	m_pActionMenu->insertSeparator ();
	m_actionStop->plug (m_pActionMenu);
	m_actionClear->plug (m_pActionMenu);
	if (m_CommandList.count () > 0)
	{
		m_pActionMenu->insertSeparator ();
		int index = m_CommandList.at ();
		for (unsigned int i = 0; i < m_CommandList.count (); i++)
			if (m_CommandList.at(i)->m_id == ID_OPTION_COMPLAIN)
				m_actionComplain->setEnabled (true);
    	else
				m_pActionMenu->insertItem (m_CommandList.at(i)->m_name, m_CommandList.at(i)->m_id);
		m_CommandList.at (index);
	}
}

void KShowMailApp::slotSetupCommands ()
{
	if (m_state != idle)
  {
    kapp->beep ();
		return;
  }
	m_state = configure;
	CommandDialog dlg (this, "Commands", &m_CommandList, m_ConfigList.current(), &m_ConfigList);
	if (dlg.exec () == QDialog::Accepted)
	{
		resetActionMenu ();
	}
	m_state = idle;
}

void KShowMailApp::slotCommand(int id)
{
	if ((id < ID_OPTION_COMPLAIN) || (m_state != idle))
  {
    kapp->beep ();
 		return;
  }

//  kdDebug() << "cmd id: " << id << endl;
	m_state = executing;
	QApplication::setOverrideCursor (Qt::waitCursor);
	if (m_ConfigList.current())
		m_CommandList.exec (id, m_ConfigList.current(), m_ConfigList.current()->current()->header(), NULL);
	else
		m_CommandList.exec (id, NULL, NULL, NULL);
	QApplication::restoreOverrideCursor ();
	m_state = idle;
}

bool KShowMailApp::event(QEvent *e)
{
  // to hide the taskbar button; any better solution ??
	if (e->type() == QEvent::Hide)
    hide ();
	
	return(KMainWindow::event(e));
}

void KShowMailApp::slotSetupFilters ()
{
	if (m_state != idle)
  {
    kapp->beep ();
		return;
  }
	m_state = configure;
	FilterDialog dlg (this, "Filters");
	dlg.exec ();
	m_state = idle;
}

