/***************************************************************************
                          kshowmail.h  -  description
                             -------------------
    begin                : Sat May  6 12:13:57 MEST 2000
    copyright            : (C) 2000-2001 by Eggert Ehmke
    email                : eggert.ehmke@berlin.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KSHOWMAIL_H
#define KSHOWMAIL_H
 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// include files for QT
#include <qlistview.h>
#include <qtoolbar.h>
#include <qtimer.h>
#include <qarray.h>
#include <qmessagebox.h>

// include files for KDE
#include <kapplication.h>
#include <kmainwindow.h>
#include <kuniqueapplication.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kaccel.h>
#include <kaboutdata.h>
#include <khelpmenu.h>
#include <kio/job.h>
#include <kio/global.h>

#include "configlist.h"
#include "commanddialog.h"
#include "alertdialog.h"
#include "showrecord.h"
#include "showlistviewitem.h"
#include "kshowmaildock.h"
#include "kfeedback.h"

// forward declaration of the Kshowmail classes
class KshowmailDoc;
class KshowmailView;

/**
  * The base class for Kshowmail application windows. It sets up the main
  * window and reads the config file as well as providing a menubar, toolbar
  * and statusbar. An instance of KshowmailView creates your center view, which is connected
  * to the window's Doc object.
  * KShowMailApp reimplements the methods that KMainWindow provides for main window handling and supports
  * full session management as well as keyboard accelerator configuration by using KAccel.
  * @see KMainWindow
  * @see KApplication
  * @see KConfig
  * @see KAccel
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
  * @version KDevelop version 0.4 code generation
  */
class KShowMailApp : public KMainWindow
{
  Q_OBJECT

  friend class KshowmailView;
  friend class KshowmailDock;
  friend class UniqueApp;

  public:
    /** construtor of KShowMailApp, calls all init functions to create the application.
     * @see initMenuBar initToolBar
     */
    KShowMailApp();
    ~KShowMailApp();
    KshowmailDoc *getDocument() const;
    static KAboutData* m_pAbout;

  protected:
    /** read general Options again and initialize all variables like the recent file list
     */
    void readOptions();
    /** sets up the statusbar for the main window by initialzing a statuslabel.
     */
    void initStatusBar();
    /** sets up the actions.
     */
    void initActions();
    /** sets up the menubar.
     */
    void initMenuBar();
    /** sets up the toolbar.
     */
    void initToolBar();
    /** initializes the document object of the main window that is connected to the view in initView().
     * @see initView();
     */
    void initDocument();
    /** creates the centerwidget of the KMainWindow instance and sets it as the view
     */
    void initView();
    /** queryClose is called by KMainWindow on each closeEvent of a window. Against the
     * default implementation (only returns true), this calles saveModified() on the document object to ask if the document shall
     * be saved if Modified; on cancel the closeEvent is rejected.
     * @see KMainWindow#queryClose
     * @see KMainWindow#closeEvent
     */
    virtual bool queryClose();
    /** queryExit is called by KMainWindow when the last window of the application is going to be closed during the closeEvent().
     * Against the default implementation that just returns true, this calls saveOptions() to save the settings of the last window's	
     * properties.
     * @see KMainWindow#queryExit
     * @see KMainWindow#closeEvent
     */
    virtual bool queryExit();

		// this overwrites QObject::timerEvent ();
    virtual void  timerEvent( QTimerEvent * );

    /**
     * operwrite QWidget::event to hide the taskbar button
     */
    virtual bool event(QEvent *e);

	private:
		bool assertPasswd (ConfigElem* pElem, bool force=false);
		void restartTimer ();
    void scanList ();
		void finalizeRefresh ();
		void finalizeDelete ();
    void scanUIDLs ();
		void startGetIndex ();	
		void startGetHeaders ();
		void startGetUIDLs ();
		void startDelete ();
		void startShowMessage ();
		void startComplain ();
		void resetActionMenu ();
		void UIDLsCont ();
		void IndexCont ();
		void HeadersCont();
  	void DeleteCont ();
  	void DownloadCont ();
	  void ComplainCont ();
    void deleteIncompleteRecords ();
    void addFeatureList(KFeedbackQuestion * question);
				
  public slots:
    /** save general Options like all bar positions and status as well as the geometry and the recent file list to the configuration
     * file
     */
    void slotSaveOptions();
    /** closes all open windows by calling close() on each memberList item until the list is empty, then quits the application.
     * If queryClose() returns false because the user canceled the saveModified() dialog, the closing breaks.
     */
    void slotFileQuit();
    /** put the marked text/object into the clipboard and remove
     *	it from the document
     */

	void slotSetupAccounts();
	void slotSetupAccount();
	void slotSetupCommands();
	void slotSetupFilters();
	void slotPop3Timeout ();
	void slotStop ();
	void slotClear ();
	void slotData (KIO::Job*, const QByteArray& data);
	void slotConnected (KIO::Job*);
	void slotRefresh();
	void slotForcedRefresh();
	void slotUIDLsResult (KIO::Job* job);
	void slotUIDLsCont (KIO::Job* job);
	void slotIndexResult (KIO::Job* job);
	void slotIndexCont (KIO::Job* job);
	void slotHeadersResult (KIO::Job* job);
	void slotHeadersCont (KIO::Job* job);
	void slotDelete();
	void slotDeleteResult (KIO::Job* job);
	void slotDeleteCont (KIO::Job* job);
	void slotComplain();
	void slotComplainResult (KIO::Job* job);
	void slotComplainCont (KIO::Job* job);
	void slotShowHeader();
	void slotShowMessage();
	void slotDownloadResult (KIO::Job* job);
	void slotDownloadCont (KIO::Job* job);
	void slotCommand(int id);
	void slotRefreshView ();

    /** toggles the toolbar
     */
    void slotToggleToolBar();
    /** toggles the statusbar
     */
    void slotToggleStatusBar();
    /** changes the statusbar contents for the standard label permanently, used to indicate current actions.
     * @param text the text that is displayed in the statusbar
     */
    void slotStatusMsg(const QString &text);
    /** changes the status message of the whole statusbar for two seconds, then restores the last status. This is used to display
     * statusbar messages that give information about actions for toolbar icons and menuentries.
     * @param text the text that is displayed in the statusbar
     */
    void slotStatusHelpMsg(const QString &text);
		void slotAccountActivated(QListViewItem*);
		void slotAlertDestroyed ();
		void slotAlertOk ();
    void slotSendFeedbackMail();

  private:
    /** the key accelerator container */
    KAccel *keyAccel;
	
    KshowmailView *m_pView;
    /** doc represents your actual document and is created only once. It keeps
     * information such as filename and does the serialization of your files.
     */
    KshowmailDoc* m_pDoc;

		ConfigList m_ConfigList;
		CommandList m_CommandList;

		AlertDialog* m_pAlertDialog;
    QMessageBox* m_pMessagebox;
  	
		KShowMailDock* m_pDockWindow;

		QTimer* m_pTimer;
		QTimer* m_pPop3Timer;

public:
		KAction*				m_actionSave;
		KAction*				m_actionQuit;
		KAction*				m_actionRefresh;
		KAction*				m_actionForcedRefresh;
		KAction*				m_actionShowHeader;
		KAction*				m_actionShowMessage;
		KAction*				m_actionDelete;
		KAction*				m_actionComplain;
		KAction*				m_actionSetupAccounts;
		KAction*				m_actionSetupAccount;
		KAction*				m_actionSetupCommands;
		KAction*				m_actionSetupFilters;
		KToggleAction*	m_actionToolBar;
		KToggleAction*	m_actionStatusBar;
		KAction*				m_actionHelp;
		KAction*				m_actionStop;
		KAction*				m_actionClear;
    KAction*        m_actionSendFeedbackMail;
		
private:
		QPopupMenu* 		m_pFileMenu;
		QPopupMenu* 		m_pSetupMenu;
		QPopupMenu* 		m_pActionMenu;
		KHelpMenu*	 		m_pHelpMenu;

		unsigned int m_nMessages;   	// messages per account
		unsigned int m_nTotalSize;		// size of all messages
		unsigned int m_nTotalCount;
		unsigned int m_nNewMail;
		unsigned int m_nDeletedMail;
		unsigned int m_nRetry;
		unsigned long m_nSecondsToGo;

		bool m_bYesAll;
		bool m_bNoAll;

		enum {idle, deleting, complaining, configure, executing,
		      waitIndexResult, waitIndexCont,
		      waitUIDLsResult, waitUIDLsCont,
		      waitHeadersResult, waitHeadersCont,
		      waitDeleteResult, waitDeleteCont,
		      waitComplainResult, waitComplainCont,
		      waitDownloadResult, waitDownloadCont} m_state;
		KIO::Job* m_pPop3;
		QCString m_data;
};

#endif // KSHOWMAIL_H
