/***************************************************************************
                          kshowmailview.cpp  -  description
                             -------------------
    begin                : Sat May  6 12:13:57 MEST 2000
    copyright            : (C) 2000-2001 by Eggert Ehmke
    email                : eggert.ehmke@berlin.de

     26 Sep 2002 - Allow for columns to be hidden. Allistar Melville
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qprinter.h>
#include <qpainter.h>
#include <qmessagebox.h>
#include <qkeycode.h>
#include <qvaluelist.h>

#include <kstandarddirs.h>
#include <klocale.h>
#include <kdebug.h>

// application specific includes
#include "kshowmailview.h"
#include "kshowmaildoc.h"
#include "kshowmail.h"
#include "showlistviewitem.h"

KshowmailView::KshowmailView(QWidget *parent, const char *name):
	QSplitter(Vertical, parent, name)
{
	m_pixOk = new QPixmap (::locate ("data", "kshowmail/pics/ok.png"));

	m_pListAccounts = new KListView (this, "accounts");
	m_pListMessages = new KListView (this, "messages");
  setBackgroundMode(PaletteBase);

  m_pListAccounts->setSelectionMode (QListView::Extended);
	m_pListAccounts->setAllColumnsShowFocus (true);
  // sort account column
  m_pListAccounts->setSorting (1);
  m_pListAccounts->setShowSortIndicator (true);
	m_pListAccounts->addColumn (i18n("Active"),    50);
	m_pListAccounts->addColumn (i18n("Account"),  100);
	m_pListAccounts->addColumn (i18n("Server"),   100);
	m_pListAccounts->addColumn (i18n("User"),     100);
	m_pListAccounts->addColumn (i18n("Messages"),  80);
	m_pListAccounts->addColumn (i18n("Size"),      80);

  m_pListMessages->setSelectionMode (QListView::Extended);
	m_pListMessages->setAllColumnsShowFocus (true);
  // sort number column
	m_pListMessages->setSorting (ShowListViewItem::_colNumber);
  m_pListMessages->setShowSortIndicator (true);
	setFocusPolicy (QWidget::StrongFocus);
	
	m_pListMessages->addColumn (i18n("Number"),   50);
	m_pListMessages->addColumn (i18n("Account"), 100);
	m_pListMessages->addColumn (i18n("From"),    200);
	m_pListMessages->addColumn (i18n("To"),      200);
	m_pListMessages->addColumn (i18n("Subject"), 350);
	m_pListMessages->addColumn (i18n("Date"),    100);
	m_pListMessages->addColumn (i18n("Size"),     80);
	m_pListMessages->addColumn (i18n("Content"),  80);
	m_pListMessages->addColumn (i18n("State"),    50);

	connect (m_pListAccounts, SIGNAL (rightButtonClicked (QListViewItem*, const QPoint &, int)),
								SLOT (slotAccountContext(QListViewItem*, const QPoint &, int)));
	connect (m_pListAccounts, SIGNAL (clicked (QListViewItem*, const QPoint &, int)),
								SLOT (slotAccountClicked(QListViewItem*, const QPoint &, int)));
	connect (m_pListMessages, SIGNAL (rightButtonClicked (QListViewItem*, const QPoint &, int)),
								SLOT (slotMessageContext(QListViewItem*, const QPoint &, int)));
	connect (m_pListMessages, SIGNAL (doubleClicked (QListViewItem*)),
								SLOT (slotMessageDoubleClicked (QListViewItem*)));
}

KshowmailView::~KshowmailView()
{
	delete m_pListAccounts;
	delete m_pListMessages;
}

void KshowmailView::readOptions (KConfig* config)
{
	config->setGroup ("Display Options");
	//splitter
	QValueList<int> size;
	size.append (config->readNumEntry ("sizeaccounts", 40));
	size.append (config->readNumEntry ("sizemessages", 50));
	setSizes (size);
	// layout
  m_pListAccounts->restoreLayout (config, "accountlayout");
  m_pListMessages->restoreLayout (config, "messagelayout");
}

void KshowmailView::saveOptions (KConfig* config)
{
//	kdDebug () << "KshowmailView::saveOptions" << endl;
	config->setGroup("Display Options");
	//splitter
	QValueList<int> size = sizes ();
	config->writeEntry ("sizeaccounts",  size [0]);
	config->writeEntry ("sizemessages",  size [1]);
	// layout
  m_pListAccounts->saveLayout (config, "accountlayout");
  m_pListMessages->saveLayout (config, "messagelayout");
}

void KshowmailView::showOrHideColumns(ConfigList *configList)
{
//modification by AM to allow the user to decide which columns to show
//26 September 2002
  QValueList<int> widths;
  int i;
  //accounts
  widths.append(50);
  widths.append(100);
  widths.append(100);
  widths.append(100);
  widths.append(80);
  widths.append(50);
  for(i=0;i<6;i++) {
    if (configList->m_shownAccountColumns[i])
    {
      if (m_pListAccounts->columnWidth(i) == 0)
        m_pListAccounts->setColumnWidth(i, widths[i]);
    }
    else
      m_pListAccounts->setColumnWidth(i, 0);
  }
  //messages
  widths.clear();
  widths.append(50);
  widths.append(100);
  widths.append(200);
  widths.append(200);
  widths.append(50);
  widths.append(100);
  widths.append(80);
  widths.append(80);
  widths.append(50);
  for(i=0;i<8;i++)
  {
    if (configList->m_shownMailColumns[i])
    {
      if (m_pListMessages->columnWidth(i) == 0)
        m_pListMessages->setColumnWidth(i, widths[i]);
    }
    else
      m_pListMessages->setColumnWidth(i, 0);
  }
}

KshowmailDoc *KshowmailView::getDocument() const
{
  KShowMailApp *theApp=(KShowMailApp *) parentWidget();

  return theApp->getDocument();
}

void KshowmailView::slotMessageContext (QListViewItem* pItem, const QPoint& point, int)
{
//  kdDebug() << "slotMessageContext" << endl;
	if (pItem)
	{
	  KShowMailApp *theApp=(KShowMailApp *) parentWidget();
		QPopupMenu *pop = new QPopupMenu ();
		theApp->m_actionShowHeader->plug (pop);
		theApp->m_actionShowMessage->plug (pop);
		theApp->m_actionDelete->plug (pop);
		theApp->m_actionComplain->plug (pop);

		pop->popup(point);

		m_pListMessages->setSelected (pItem, true);
	}
}

void KshowmailView::slotAccountContext (QListViewItem* pItem, const QPoint& point, int)
{
	if (pItem)
	{
	  KShowMailApp *theApp=(KShowMailApp *) parentWidget();
		QPopupMenu *pop = new QPopupMenu ();
		theApp->m_actionRefresh->plug (pop);
		theApp->m_actionSetupAccount->plug (pop);

		pop->popup(point);

		m_pListAccounts->setSelected (pItem, true);
	}
}

void KshowmailView::slotAccountClicked (QListViewItem* pItem, const QPoint&, int col)
{
	if (col == 0)
	{
		KShowMailApp *theApp=(KShowMailApp *) parentWidget();
		QPtrListIterator<ConfigElem> it(theApp->m_ConfigList);
		// this is = not == !!!
		while (ConfigElem* pElem = it())
		{
			if (pElem->m_pViewItem == pItem)
			{
				pElem->m_bActive = !pElem->m_bActive;
				if (pElem->m_bActive)
					pItem->setPixmap (0, *m_pixOk);
				else
					pItem->setPixmap (0, 0);
				emit signalActiveChanged ();
				return;
			}
		}
	}
}

void KshowmailView::slotMessageDoubleClicked (QListViewItem* pItem)
{
	if (pItem)
	{
		m_pListMessages->setSelected (pItem, true);

  	KShowMailApp *theApp=(KShowMailApp *) parentWidget();
		theApp->slotShowHeader ();
	}
}
