/***************************************************************************
                          optiondialog.cpp  -  description
                             -------------------
    begin                : Fri Oct 26 2001
    copyright            : (C) 2001 by Eggert Ehmke
    email                : eggert.ehmke@berlin.de

    26 Sep 2002 - Allow for columns to be hidden. Allistar Melville
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qcheckbox.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qfileinfo.h>
#include <qfiledialog.h>
#include <qpushbutton.h>

#include <kmessagebox.h>
#include <klocale.h>
#include <knuminput.h>
#include <kdebug.h>

#include "optiondialog.h"
#include "serverdialog.h"
#include "configlist.h"
#include "configelem.h"
#include "commandlist.h"

OptionDialog::OptionDialog(QWidget *parent, const char *name, ConfigList* config, CommandList* cmd):
	OptionDlg (parent,name,true),
	m_ConfigList (*config),
	m_pConfigList (config),
	m_pCmdList (cmd)
{
	m_CheckSavePassword->setChecked (m_pConfigList->m_bSavePassword);
	m_CheckConfirmDelete->setChecked (m_pConfigList->m_bConfirmDelete);
	m_SpinInitTimer->setValue (m_pConfigList->m_nInitTimer);
	m_SpinIntervalTimer->setValue (m_pConfigList->m_nIntervalTimer);
  m_SpinPop3Timer->setValue (m_pConfigList->m_nPop3Timer);
  m_CheckBeep->setChecked (m_pConfigList->m_bBeep);
  m_CheckSound->setChecked (m_pConfigList->m_bSound);
	m_ButtonSelectSoundFile->setEnabled (m_pConfigList->m_bSound);
	m_EditSoundFile->setEnabled (m_pConfigList->m_bSound);
	m_ButtonSoundTest->setEnabled (m_pConfigList->m_bSound);
  m_EditSoundFile->setText (m_pConfigList->m_strSoundFile);
  m_CheckStartMinimized->setChecked (m_pConfigList->m_bStartMinimized);
  m_CheckTerminateNoMail->setChecked (m_pConfigList->m_bTerminate);
  m_CheckMinimizeNoMail->setChecked (m_pConfigList->m_bMinimize);
  m_CheckKeepNewMail->setChecked (m_pConfigList->m_bKeepNew);
  m_CheckShowMessagebox->setChecked (m_pConfigList->m_bShowMessage);
  m_CheckShowMainWindow->setChecked (m_pConfigList->m_bShowMainWindow);
  m_CheckExternalProg->setChecked (m_pConfigList->m_bExternalProg);
	m_ButtonExternalProgTest->setEnabled (m_pConfigList->m_bExternalProg);
	m_ComboExternalProg->setEnabled (m_pConfigList->m_bExternalProg);
	QLabel_5->setEnabled (m_pConfigList->m_bExternalProg);
  m_SpinMaxSize->setValue (m_pConfigList->m_nMaxMailSize);
	m_ConfigList.setCombo (m_ComboAccount);
	m_pCmdList->setCombo (m_ComboExternalProg, m_pConfigList->m_nExternalProg);
	//Modifications made by AM (account columns)
	m_CheckActive->setChecked(m_pConfigList->m_shownAccountColumns[0]);
	m_CheckAccount1->setChecked(m_pConfigList->m_shownAccountColumns[1]);
	m_CheckServer->setChecked(m_pConfigList->m_shownAccountColumns[2]);
	m_CheckUser->setChecked(m_pConfigList->m_shownAccountColumns[3]);
	m_CheckMessages->setChecked(m_pConfigList->m_shownAccountColumns[4]);
	m_CheckSize1->setChecked(m_pConfigList->m_shownAccountColumns[5]);
	//Modifications made by AM (email columns)
	m_CheckNumber->setChecked(m_pConfigList->m_shownMailColumns[0]);
	m_CheckAccount2->setChecked(m_pConfigList->m_shownMailColumns[1]);
	m_CheckFrom->setChecked(m_pConfigList->m_shownMailColumns[2]);
	m_CheckTo->setChecked(m_pConfigList->m_shownMailColumns[3]);
	m_CheckSubject->setChecked(m_pConfigList->m_shownMailColumns[4]);
	m_CheckDate->setChecked(m_pConfigList->m_shownMailColumns[5]);
	m_CheckSize2->setChecked(m_pConfigList->m_shownMailColumns[6]);
	m_CheckContent->setChecked(m_pConfigList->m_shownMailColumns[7]);
	m_CheckState->setChecked(m_pConfigList->m_shownMailColumns[8]);
  // add HTML formatting
  m_checkHTML->setChecked(m_pConfigList->m_bAllowHTML);
  _checkConfirmClose->setChecked(m_pConfigList->m_bConfirmClose);
}

OptionDialog::~OptionDialog(){
}

void OptionDialog::slotButtonAddClicked()
{
	ConfigElem* pElem = new ConfigElem ();
	ServerDialog dlg (this, "Server Options", pElem, &m_ConfigList);
	if (dlg.exec () == Accepted)
	{
		m_ConfigList.inSort (pElem);
		m_ComboAccount->insertItem (pElem->m_strAccount, m_ConfigList.at ());
		m_ComboAccount->setCurrentItem (m_ConfigList.at ());
	}
	else
		delete pElem;
}

void OptionDialog::slotButtonEditClicked()
{
	ConfigElem* pElem = m_ConfigList.current ();
	if (pElem)
	{
		ServerDialog dlg (this, "Server Options", pElem, NULL);
		dlg.exec ();
	}
}

void OptionDialog::slotButtonCopyClicked()
{
	if (m_ConfigList.current())
	{
		ConfigElem* pElem = new ConfigElem (m_ConfigList.current ());
		ServerDialog dlg (this, "Server Options", pElem, &m_ConfigList);
		if (dlg.exec () == Accepted)
		{
			pElem->m_pshowrecord = new ShowRecord ();
			m_ConfigList.inSort (pElem);
			m_ComboAccount->insertItem (pElem->m_strAccount, m_ConfigList.at ());
			m_ComboAccount->setCurrentItem (m_ConfigList.at ());
		}
		else
			delete pElem;
	}
}

void OptionDialog::slotButtonDeleteClicked()
{
	if (m_ConfigList.current())
	{
		if (KMessageBox::warningYesNo (this, i18n("Account %1 will be deleted.\nAre you sure ?").arg(m_ConfigList.current()->m_strAccount)) == KMessageBox::Yes)
		{
			int index = m_ConfigList.at ();
			m_ConfigList.remove (index);
			m_ComboAccount->removeItem (index);
			m_ComboAccount->setCurrentItem (m_ConfigList.at ());
		}
	}
}

void OptionDialog::slotButtonOkClicked()
{
	m_ConfigList.m_bSavePassword = m_CheckSavePassword->isChecked ();
	m_ConfigList.m_bConfirmDelete = m_CheckConfirmDelete->isChecked ();
	m_ConfigList.m_nInitTimer = m_SpinInitTimer->value ();
	m_ConfigList.m_nIntervalTimer = m_SpinIntervalTimer->value ();
  m_ConfigList.m_nPop3Timer = m_SpinPop3Timer->value ();
	m_ConfigList.m_nMaxMailSize = m_SpinMaxSize->value ();
	m_ConfigList.m_bBeep = m_CheckBeep->isChecked ();
	m_ConfigList.m_bSound = m_CheckSound->isChecked ();
	m_ConfigList.m_strSoundFile = m_EditSoundFile->text ();
	m_ConfigList.m_bStartMinimized = m_CheckStartMinimized->isChecked ();
	m_ConfigList.m_bTerminate = m_CheckTerminateNoMail->isChecked ();
	m_ConfigList.m_bMinimize = m_CheckMinimizeNoMail->isChecked ();
	m_ConfigList.m_bKeepNew = m_CheckKeepNewMail->isChecked ();
	m_ConfigList.m_bShowMessage = m_CheckShowMessagebox->isChecked ();
	m_ConfigList.m_bShowMainWindow = m_CheckShowMainWindow->isChecked ();
	m_ConfigList.m_nExternalProg = m_ComboExternalProg->currentItem ();
	kdDebug () << "ext prog = " << m_ConfigList.m_nExternalProg << endl;
	kdDebug () << "count = " << m_ConfigList.count() << endl;
	if ((m_ConfigList.m_nExternalProg >= 0) && (m_ConfigList.count() > 0))
		m_ConfigList.m_bExternalProg = m_CheckExternalProg->isChecked ();
	else
		m_ConfigList.m_bExternalProg = false;
	//modifications by AM (hiding columns)
	m_ConfigList.m_shownAccountColumns[0] = m_CheckActive->isChecked();
	m_ConfigList.m_shownAccountColumns[1] = m_CheckAccount1->isChecked();
	m_ConfigList.m_shownAccountColumns[2] = m_CheckServer->isChecked();
	m_ConfigList.m_shownAccountColumns[3] = m_CheckUser->isChecked();
	m_ConfigList.m_shownAccountColumns[4] = m_CheckMessages->isChecked();
	m_ConfigList.m_shownAccountColumns[5] = m_CheckSize1->isChecked();
	m_ConfigList.m_shownMailColumns[0] = m_CheckNumber->isChecked();
	m_ConfigList.m_shownMailColumns[1] = m_CheckAccount2->isChecked();
	m_ConfigList.m_shownMailColumns[2] = m_CheckFrom->isChecked();
	m_ConfigList.m_shownMailColumns[3] = m_CheckTo->isChecked();
	m_ConfigList.m_shownMailColumns[4] = m_CheckSubject->isChecked();
	m_ConfigList.m_shownMailColumns[5] = m_CheckDate->isChecked();
	m_ConfigList.m_shownMailColumns[6] = m_CheckSize2->isChecked();
	m_ConfigList.m_shownMailColumns[7] = m_CheckContent->isChecked();
	m_ConfigList.m_shownMailColumns[8] = m_CheckState->isChecked();
  m_ConfigList.m_bAllowHTML = m_checkHTML->isChecked();
  m_ConfigList.m_bConfirmClose = _checkConfirmClose->isChecked();
	*m_pConfigList = m_ConfigList;
}

void OptionDialog::slotAccountActivated (int index)
{
	m_ConfigList.at (index); // will set current item
}

void OptionDialog::slotSavePasswordToggled ()
{
	if (m_CheckSavePassword->isChecked ())
	{
		if (KMessageBox::warningYesNo (this, i18n("Password will be stored as scrambled text.\n"
		                                     "It might be possible to decipher it.\n"
		                                     "Are you sure ?\n"),
                                         QString::null,
                                         KStdGuiItem::yes(),
                                         KStdGuiItem::no(),
                                         "savepasswdaskagain") != KMessageBox::Yes)
			m_CheckSavePassword->setChecked (false);
	}
}

void OptionDialog::slotButtonSelectSoundFile ()
{
	QStringList filters;
	filters.append (i18n("Wave files (*.wav)"));
	filters.append (i18n("All files (*)"));
	QFileInfo info (m_EditSoundFile->text ());
	QString dir = info.dirPath ();
	QFileDialog dlg (this, "Sound File", TRUE);
	dlg.setCaption (i18n("Select Sound File"));
	dlg.setDir (dir);
	dlg.setFilters (filters);
	dlg.setMode (QFileDialog::ExistingFile);
  if (dlg.exec () == QDialog::Accepted)
	 	m_EditSoundFile->setText (dlg.selectedFile());
}

void OptionDialog::slotMinimizeNoMailClicked ()
{
	if (m_CheckMinimizeNoMail->isChecked ())
		m_CheckTerminateNoMail->setChecked (false);
}

void OptionDialog::slotTerminateNoMailClicked ()
{
	if (m_CheckTerminateNoMail->isChecked ())
		m_CheckMinimizeNoMail->setChecked (false);
}

void OptionDialog::slotSoundTest ()
{
	m_pConfigList->playSound (m_EditSoundFile->text ());
}

void OptionDialog::slotExternalProgTest ()
{
	m_pCmdList->exec (m_ComboExternalProg->currentItem (), m_pConfigList->current ());
}


