/***************************************************************************
                          showrecord.cpp  -  description
                             -------------------
    begin                : Thu Dec 28 2000
    copyright            : (C) 2000-2001 by Eggert Ehmke
    email                : eggert.ehmke@berlin.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qdatetime.h>

#include <klocale.h>
#include <kglobal.h>
#include <kdebug.h>

#include <mimelib/datetime.h>

#include "showrecordelem.h"
#include "showlistviewitem.h"
#include "decodeRFC2047.h"
#include "filter.h"
#include "logger.h"

ShowRecordElem::ShowRecordElem ()
: m_from ("???"), m_subject ("???"), m_pItem (NULL), m_size (0), m_deleted (false), m_new(false), m_pFilter(NULL)
{}

ShowRecordElem::ShowRecordElem (int size)
: m_from ("???"), m_subject ("???"), m_pItem (NULL), m_size (size), m_deleted (false), m_new(false), m_pFilter(NULL)
{}

void ShowRecordElem::scanHeader (const QString& item ,QCString& headerline)
{
	/* get e.g. the "From:" line, starting with cr,lf,"From:" and ending
           with a carriage return */

  QString searchstring (QString ("\r\n%1:").arg(item));

	int pos1 = m_header.find (searchstring, 0, FALSE);
	int pos2 = m_header.find ('\r', pos1+2);

	if (pos1 >= 0)
	{
		headerline = m_header.mid (pos1 + searchstring.length (), pos2 - pos1 - searchstring.length ());
	}
	else
	{
		/* there was no headerline for the item */

		headerline = "";
	}
}

void ShowRecordElem::setHeader (const QString& header)
{
  m_header = header;

  QCString from;
	scanHeader ("From" , from);
  from = from.simplifyWhiteSpace ();
	setFrom (from);

  QCString to;
	scanHeader ("To" , to);
  to = to.simplifyWhiteSpace ();
	setTo (to);

  QCString subject;
	scanHeader ("Subject" , subject);
  subject = subject.simplifyWhiteSpace ();
	setSubject (subject);

	QCString date;
	scanHeader ("Date" , date);
	setDate (date);

	QCString content;
	scanHeader ("Content-Type", content);
	content = content.simplifyWhiteSpace ();
	if (content.find ("text", 0, false) == 0)
		content = "";
	else if (content.find ("multi", 0, false) == 0)
		content.remove (content.find (';'),99);
	setContent (content);
}

void ShowRecordElem::setDate (const QString& date)
{
  m_strDate = date;
  DwDateTime dwDate;
  dwDate.FromString(date);
  dwDate.Parse();
  m_unixDate.setTime_t (dwDate.AsUnixTime());
}

void ShowRecordElem::setDeleted ()
{
  m_deleted = true;
  if (m_pFilter)
  {
    m_pFilter->_counter++;
    Logger() << i18n("Time") << ": " << KGlobal::locale()->formatDateTime(QDateTime::currentDateTime(), true, true) << endl;
    Logger() << i18n("Filter") << ": " << m_pFilter->toString() << endl;
    Logger() << i18n("Deleted") << ": "
             << i18n("From") << " = " << from() << "; "
             << i18n("To") << " = " << to() << "; "
             << i18n("Subject") << " = " << subject() << endl << endl;
    Logger::flush();
  }
}


QString ShowRecordElem::from () const
{
  return decodeRFC2047 (m_from);
}

QString ShowRecordElem::to () const
{
  return decodeRFC2047 (m_to);
}

QString ShowRecordElem::subject () const
{
  return decodeRFC2047 (m_subject);
}

QString ShowRecordElem::date () const
{
	return KGlobal::locale()->formatDateTime(m_unixDate, true, true);
}

QString ShowRecordElem::strUnixTime () const
{
	return m_unixDate.toString (Qt::ISODate);
}

QString ShowRecordElem::strSize () const
{
  // format size right aligned to allow sorting
  return QString ("%1").arg(m_size, 8);
}

QString ShowRecordElem::state () const
{
	if (m_new)
		return i18n ("new");
	else
		return i18n ("old");
}

void ShowRecordElem::saveOptions (QDomDocument& doc, QDomElement& parent)
{
	QString hdr (QString ("message%1").arg(m_nNumber));

	QDomElement elem = doc.createElement(hdr);
	elem.setAttribute ("number", m_nNumber);
	elem.setAttribute ("size", m_size);
	elem.setAttribute ("uidl", m_uidl);

	QDomElement subelem = doc.createElement ("header");
	subelem.appendChild (doc.createTextNode (m_header));
	elem.appendChild (subelem);
	
	parent.appendChild (elem);
}

void ShowRecordElem::readOptions (QDomElement& elem)
{
//	kdDebug () << "ShowRecordElem::readOptions: name = " << elem.tagName() << endl;
	setNumber (elem.attribute ("number").toInt());
	setSize (elem.attribute ("size").toInt());
	setUIDL (elem.attribute ("uidl"));
	QDomElement subelem = elem.namedItem ("header").toElement();
	setHeader (subelem.text ());
}

void ShowRecordElem::applyFilters ()
{
  FilterElem* filter = Filter::matches (this);
  if (filter)
  {
    m_pItem->setSelected (true);
    setFilter (filter);
  }  
}
